@@ -133,6 +133,7 @@ static const char* authors[] = {
 	N_("Ben Bell"),
 	N_("Sakari Bergen"),
 	N_("Christian Borss"),
+	N_("Thomas Brand"),
 	N_("Chris Cannam"),
 	N_("Jeremy Carter"),
 	N_("Jesse Chappell"),
@@ -170,6 +171,7 @@ static const char* authors[] = {
 	N_("Nick Mainsbridge"),
 	N_("Tim Mayberry"),
 	N_("Doug Mclain"),
+	N_("Johannes Mueller"),
 	N_("Todd Naugle"),
 	N_("André Nusser"),
 	N_("Bent Bisballe Nyeng"),
@@ -184,6 +186,7 @@ static const char* authors[] = {
 	N_("Andreas Ruge"),
 	N_("Sampo Savolainen"),
 	N_("Rodrigo Severo"),
+	N_("Daniel Sheeler"),
 	N_("Per Sigmond"),
 	N_("Lincoln Spiteri"),
 	N_("Mike Start"),
@@ -221,6 +224,7 @@ static const char* translators[] = {
 	N_("Czech:\n\t Pavel Fric <pavelfric@seznam.cz>\n"),
 	N_("Norwegian:\n\t Eivind Ødegård\n"),
 	N_("Chinese:\n\t Rui-huai Zhang <zrhzrh@mail.ustc.edu.cn>\n"),
+	N_("Japanese:\n\t Hiroki Inagaki <hiroki.ingk@gmail.com>\n"),
 	0
 };
 
@@ -625,7 +629,7 @@ About::About ()
 #endif
 
 	set_translator_credits (t);
-	set_copyright (_("Copyright (C) 1999-2016 Paul Davis\n"));
+	set_copyright (_("Copyright (C) 1999-2017 Paul Davis\n"));
 	set_license (gpl);
 	set_name (X_("Ardour"));
 	set_website (X_("http://ardour.org/"));
