@@ -27,6 +27,7 @@
 #include "ardour/plugin_manager.h"
 #include "ardour/route.h"
 #include "ardour/session.h"
+#include "ardour/system_exec.h"
 
 #include "LuaBridge/LuaBridge.h"
 
@@ -34,11 +35,14 @@
 #include "ardour_ui.h"
 #include "public_editor.h"
 #include "region_selection.h"
+#include "luadialog.h"
 #include "luainstance.h"
 #include "luasignal.h"
 #include "marker.h"
+#include "region_view.h"
 #include "processor_box.h"
 #include "time_axis_view.h"
+#include "time_axis_view_item.h"
 #include "selection.h"
 #include "script_selector.h"
 #include "timers.h"
@@ -308,6 +312,11 @@ class PangoLayout {
 			pango_cairo_show_layout (c->cobj (), _layout->gobj());
 		}
 
+		void layout_cairo_path (Cairo::Context* c) {
+			pango_cairo_update_layout (c->cobj (), _layout->gobj());
+			pango_cairo_layout_path (c->cobj (), _layout->gobj());
+		}
+
 	private:
 		Glib::RefPtr<Pango::Layout> _layout;
 };
@@ -359,6 +368,60 @@ namespace LuaMixer {
 	}
 
 };
+////////////////////////////////////////////////////////////////////////////////
+
+static PBD::ScopedConnectionList _luaexecs;
+
+static void reaper (ARDOUR::SystemExec* x)
+{
+	delete x;
+}
+
+static int
+lua_forkexec (lua_State *L)
+{
+	int argc = lua_gettop (L);
+	if (argc == 0) {
+		return luaL_argerror (L, 1, "invalid number of arguments, forkexec (command, ...)");
+	}
+	// args are free()ed in ~SystemExec
+	char** args = (char**) malloc ((argc + 1) * sizeof(char*));
+	for (int i = 0; i < argc; ++i) {
+		args[i] = strdup (luaL_checkstring (L, i + 1));
+	}
+	args[argc] = 0;
+
+	ARDOUR::SystemExec* x = new ARDOUR::SystemExec (args[0], args);
+	x->Terminated.connect (_luaexecs, MISSING_INVALIDATOR, boost::bind (&reaper, x), gui_context());
+
+	if (x->start()) {
+		reaper (x);
+		luabridge::Stack<bool>::push (L, false);
+		return -1;
+	} else {
+		luabridge::Stack<bool>::push (L, false);
+	}
+	return 1;
+}
+
+#ifndef PLATFORM_WINDOWS
+static int
+lua_exec (std::string cmd)
+{
+	// args are free()ed in ~SystemExec
+	char** args = (char**) malloc (4 * sizeof(char*));
+	args[0] = strdup ("/bin/sh");
+	args[1] = strdup ("-c");
+	args[2] = strdup (cmd.c_str());
+	args[3] = 0;
+	ARDOUR::SystemExec x ("/bin/sh", args);
+	if (x.start()) {
+		return -1;
+	}
+	x.wait ();
+	return 0;
+}
+#endif
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -368,6 +431,7 @@ namespace LuaMixer {
 using namespace ARDOUR;
 
 PBD::Signal0<void> LuaInstance::LuaTimerDS;
+PBD::Signal0<void> LuaInstance::SetSession;
 
 void
 LuaInstance::register_hooks (lua_State* L)
@@ -389,6 +453,10 @@ LuaInstance::register_hooks (lua_State* L)
 		.beginStdBitSet <LuaSignal::LAST_SIGNAL> ("Set")
 		.endClass()
 		.endNamespace ();
+
+#if 0 // Dump size -> libs/ardour/luabindings.cc
+	printf ("LuaInstance: registered %d signals\n", LuaSignal::LAST_SIGNAL);
+#endif
 }
 
 void
@@ -491,6 +559,7 @@ LuaInstance::bind_cairo (lua_State* L)
 		.addFunction ("get_text", &LuaCairo::PangoLayout::get_text)
 		.addFunction ("set_text", &LuaCairo::PangoLayout::set_text)
 		.addFunction ("show_in_cairo_context", &LuaCairo::PangoLayout::show_in_cairo_context)
+		.addFunction ("layout_cairo_path", &LuaCairo::PangoLayout::layout_cairo_path)
 		.addFunction ("set_markup", &LuaCairo::PangoLayout::set_markup)
 		.addFunction ("set_width", &LuaCairo::PangoLayout::set_width)
 		.addFunction ("set_ellipsize", &LuaCairo::PangoLayout::set_ellipsize)
@@ -540,6 +609,51 @@ LuaInstance::bind_cairo (lua_State* L)
 
 }
 
+void
+LuaInstance::bind_dialog (lua_State* L)
+{
+	luabridge::getGlobalNamespace (L)
+		.beginNamespace ("LuaDialog")
+
+		.beginClass <LuaDialog::Message> ("Message")
+		.addConstructor <void (*) (std::string const&, std::string const&, LuaDialog::Message::MessageType, LuaDialog::Message::ButtonType)> ()
+		.addFunction ("run", &LuaDialog::Message::run)
+		.endClass ()
+
+		.beginClass <LuaDialog::Dialog> ("Dialog")
+		.addConstructor <void (*) (std::string const&, luabridge::LuaRef)> ()
+		.addCFunction ("run", &LuaDialog::Dialog::run)
+		.endClass ()
+
+		/* enums */
+		.beginNamespace ("MessageType")
+		.addConst ("Info", LuaDialog::Message::Info)
+		.addConst ("Warning", LuaDialog::Message::Warning)
+		.addConst ("Question", LuaDialog::Message::Question)
+		.addConst ("Error", LuaDialog::Message::Error)
+		.endNamespace ()
+
+		.beginNamespace ("ButtonType")
+		.addConst ("OK", LuaDialog::Message::OK)
+		.addConst ("Close", LuaDialog::Message::Close)
+		.addConst ("Cancel", LuaDialog::Message::Cancel)
+		.addConst ("Yes_No", LuaDialog::Message::Yes_No)
+		.addConst ("OK_Cancel", LuaDialog::Message::OK_Cancel)
+		.endNamespace ()
+
+		.beginNamespace ("Response")
+		.addConst ("OK", 0)
+		.addConst ("Cancel", 1)
+		.addConst ("Close", 2)
+		.addConst ("Yes", 3)
+		.addConst ("No", 4)
+		.addConst ("None", -1)
+		.endNamespace ()
+
+		.endNamespace ();
+
+}
+
 void
 LuaInstance::register_classes (lua_State* L)
 {
@@ -549,7 +663,7 @@ LuaInstance::register_classes (lua_State* L)
 	LuaBindings::osc (L);
 
 	bind_cairo (L);
-	register_hooks (L);
+	bind_dialog (L);
 
 	luabridge::getGlobalNamespace (L)
 		.beginNamespace ("ArdourUI")
@@ -567,17 +681,42 @@ LuaInstance::register_classes (lua_State* L)
 		.addFunction ("_type", &ArdourMarker::type)
 		.endClass ()
 
-#if 0
 		.beginClass <AxisView> ("AxisView")
 		.endClass ()
+
 		.deriveClass <TimeAxisView, AxisView> ("TimeAxisView")
 		.endClass ()
-		.deriveClass <RouteTimeAxisView, TimeAxisView> ("RouteTimeAxisView")
+
+		.deriveClass <StripableTimeAxisView, TimeAxisView> ("StripableTimeAxisView")
 		.endClass ()
-#endif
+
+		.beginClass <Selectable> ("Selectable")
+		.endClass ()
+
+		.deriveClass <TimeAxisViewItem, Selectable> ("TimeAxisViewItem")
+		.endClass ()
+
+		.deriveClass <RegionView, TimeAxisViewItem> ("RegionView")
+		.endClass ()
+
+		.deriveClass <RouteUI, Selectable> ("RouteUI")
+		.endClass ()
+
+		.deriveClass <RouteTimeAxisView, RouteUI> ("RouteTimeAxisView")
+		.addCast<StripableTimeAxisView> ("to_stripabletimeaxisview")
+		.addCast<TimeAxisView> ("to_timeaxisview") // deprecated
+		.endClass ()
+
+		// std::list<Selectable*>
+		.beginStdCPtrList <Selectable> ("SelectionList")
+		.endClass ()
+
+		// std::list<TimeAxisView*>
+		.beginStdCPtrList <TimeAxisView> ("TrackViewStdList")
+		.endClass ()
+
 
 		.beginClass <RegionSelection> ("RegionSelection")
-		.addFunction ("clear_all", &RegionSelection::clear_all)
 		.addFunction ("start", &RegionSelection::start)
 		.addFunction ("end_frame", &RegionSelection::end_frame)
 		.addFunction ("n_midi_regions", &RegionSelection::n_midi_regions)
@@ -593,8 +732,9 @@ LuaInstance::register_classes (lua_State* L)
 		.deriveClass <MarkerSelection, std::list<ArdourMarker*> > ("MarkerSelection")
 		.endClass ()
 
-		.beginClass <TrackViewList> ("TrackViewList")
-		.addFunction ("routelist", &TrackViewList::routelist) // XXX check windows binding (libardour)
+		.deriveClass <TrackViewList, std::list<TimeAxisView*> > ("TrackViewList")
+		.addFunction ("contains", &TrackViewList::contains)
+		.addFunction ("routelist", &TrackViewList::routelist)
 		.endClass ()
 
 		.deriveClass <TrackSelection, TrackViewList> ("TrackSelection")
@@ -640,6 +780,8 @@ LuaInstance::register_classes (lua_State* L)
 		.addFunction ("get_cut_buffer", &PublicEditor::get_cut_buffer)
 		.addRefFunction ("get_selection_extents", &PublicEditor::get_selection_extents)
 
+		.addFunction ("set_selection", &PublicEditor::set_selection)
+
 		.addFunction ("play_selection", &PublicEditor::play_selection)
 		.addFunction ("play_with_preroll", &PublicEditor::play_with_preroll)
 		.addFunction ("maybe_locate_with_edit_preroll", &PublicEditor::maybe_locate_with_edit_preroll)
@@ -670,21 +812,25 @@ LuaInstance::register_classes (lua_State* L)
 		.addFunction ("get_current_zoom", &PublicEditor::get_current_zoom)
 		.addFunction ("reset_zoom", &PublicEditor::reset_zoom)
 
-#if 0 // These need TimeAxisView* which isn't exposed, yet
-		.addFunction ("playlist_selector", &PublicEditor::playlist_selector)
 		.addFunction ("clear_playlist", &PublicEditor::clear_playlist)
 		.addFunction ("new_playlists", &PublicEditor::new_playlists)
 		.addFunction ("copy_playlists", &PublicEditor::copy_playlists)
 		.addFunction ("clear_playlists", &PublicEditor::clear_playlists)
-#endif
 
 		.addFunction ("select_all_tracks", &PublicEditor::select_all_tracks)
 		.addFunction ("deselect_all", &PublicEditor::deselect_all)
-#if 0
+
+#if 0 // TimeAxisView&  can't be bound (pure virtual fn)
 		.addFunction ("set_selected_track", &PublicEditor::set_selected_track)
 		.addFunction ("set_selected_mixer_strip", &PublicEditor::set_selected_mixer_strip)
-		.addFunction ("hide_track_in_display", &PublicEditor::hide_track_in_display)
+		.addFunction ("ensure_time_axis_view_is_visible", &PublicEditor::ensure_time_axis_view_is_visible)
 #endif
+		.addFunction ("hide_track_in_display", &PublicEditor::hide_track_in_display)
+		.addFunction ("show_track_in_display", &PublicEditor::show_track_in_display)
+		.addFunction ("set_visible_track_count", &PublicEditor::set_visible_track_count)
+		.addFunction ("fit_selection", &PublicEditor::fit_selection)
+
+		.addFunction ("regionview_from_region", &PublicEditor::regionview_from_region)
 		.addFunction ("set_stationary_playhead", &PublicEditor::set_stationary_playhead)
 		.addFunction ("stationary_playhead", &PublicEditor::stationary_playhead)
 		.addFunction ("set_follow_playhead", &PublicEditor::set_follow_playhead)
@@ -695,7 +841,6 @@ LuaInstance::register_classes (lua_State* L)
 		.addFunction ("current_page_samples", &PublicEditor::current_page_samples)
 		.addFunction ("visible_canvas_height", &PublicEditor::visible_canvas_height)
 		.addFunction ("temporal_zoom_step", &PublicEditor::temporal_zoom_step)
-		//.addFunction ("ensure_time_axis_view_is_visible", &PublicEditor::ensure_time_axis_view_is_visible)
 		.addFunction ("override_visible_track_count", &PublicEditor::override_visible_track_count)
 
 		.addFunction ("scroll_tracks_down_line", &PublicEditor::scroll_tracks_down_line)
@@ -727,15 +872,15 @@ LuaInstance::register_classes (lua_State* L)
 		.addFunction ("set_video_timeline_height", &PublicEditor::set_video_timeline_height)
 
 #if 0
-		.addFunction ("get_route_view_by_route_id", &PublicEditor::get_route_view_by_route_id)
 		.addFunction ("get_equivalent_regions", &PublicEditor::get_equivalent_regions)
-
-		.addFunction ("axis_view_from_route", &PublicEditor::axis_view_from_route)
-		.addFunction ("axis_views_from_routes", &PublicEditor::axis_views_from_routes)
-		.addFunction ("get_track_views", &PublicEditor::get_track_views)
 		.addFunction ("drags", &PublicEditor::drags)
 #endif
 
+		.addFunction ("get_stripable_time_axis_by_id", &PublicEditor::get_stripable_time_axis_by_id)
+		.addFunction ("get_track_views", &PublicEditor::get_track_views)
+		.addFunction ("rtav_from_route", &PublicEditor::rtav_from_route)
+		.addFunction ("axis_views_from_routes", &PublicEditor::axis_views_from_routes)
+
 		.addFunction ("center_screen", &PublicEditor::center_screen)
 
 		.addFunction ("get_smart_mode", &PublicEditor::get_smart_mode)
@@ -760,6 +905,7 @@ LuaInstance::register_classes (lua_State* L)
 #endif
 
 		.addFunction ("access_action", &PublicEditor::access_action)
+		.addFunction ("set_toggleaction", &PublicEditor::set_toggleaction)
 		.endClass ()
 
 		/* ArdourUI enums */
@@ -777,7 +923,21 @@ LuaInstance::register_classes (lua_State* L)
 		.addConst ("PunchOut", ArdourMarker::Type(ArdourMarker::PunchOut))
 		.endNamespace ()
 
-		.endNamespace (); // end ArdourUI
+		.beginNamespace ("SelectionOp")
+		.addConst ("Toggle", Selection::Operation(Selection::Toggle))
+		.addConst ("Set", Selection::Operation(Selection::Set))
+		.addConst ("Extend", Selection::Operation(Selection::Extend))
+		.addConst ("Add", Selection::Operation(Selection::Add))
+		.endNamespace ()
+
+		.endNamespace () // end ArdourUI
+
+		.beginNamespace ("os")
+#ifndef PLATFORM_WINDOWS
+		.addFunction ("execute", &lua_exec)
+#endif
+		.addCFunction ("forkexec", &lua_forkexec)
+		.endNamespace ();
 
 	// Editing Symbols
 
@@ -833,6 +993,13 @@ LuaInstance::instance ()
 	return _instance;
 }
 
+void
+LuaInstance::destroy_instance ()
+{
+	delete _instance;
+	_instance = 0;
+}
+
 LuaInstance::LuaInstance ()
 {
 	lua.Print.connect (&_lua_print);
@@ -844,6 +1011,7 @@ LuaInstance::LuaInstance ()
 LuaInstance::~LuaInstance ()
 {
 	delete _lua_call_action;
+	delete _lua_render_icon;
 	delete _lua_add_action;
 	delete _lua_del_action;
 	delete _lua_get_action;
@@ -857,24 +1025,31 @@ LuaInstance::~LuaInstance ()
 void
 LuaInstance::init ()
 {
+	lua.sandbox (false);
 	lua.do_command (
 			"function ScriptManager ()"
-			"  local self = { scripts = {}, instances = {} }"
+			"  local self = { scripts = {}, instances = {}, icons = {} }"
 			""
 			"  local remove = function (id)"
 			"   self.scripts[id] = nil"
 			"   self.instances[id] = nil"
+			"   self.icons[id] = nil"
 			"  end"
 			""
-			"  local addinternal = function (i, n, s, f, a)"
+			"  local addinternal = function (i, n, s, f, c, a)"
 			"   assert(type(i) == 'number', 'id must be numeric')"
 			"   assert(type(n) == 'string', 'Name must be string')"
 			"   assert(type(s) == 'string', 'Script must be string')"
 			"   assert(type(f) == 'function', 'Factory is a not a function')"
 			"   assert(type(a) == 'table' or type(a) == 'nil', 'Given argument is invalid')"
-			"   self.scripts[i] = { ['n'] = n, ['s'] = s, ['f'] = f, ['a'] = a }"
-			"   local env = _ENV;  env.f = nil env.debug = nil os.exit = nil require = nil dofile = nil loadfile = nil package = nil"
+			"   self.scripts[i] = { ['n'] = n, ['s'] = s, ['f'] = f, ['a'] = a, ['c'] = c }"
+			"   local env = _ENV; env.f = nil env.io = nil"
 			"   self.instances[i] = load (string.dump(f, true), nil, nil, env)(a)"
+			"   if type(c) == 'function' then"
+			"     self.icons[i] = load (string.dump(c, true), nil, nil, env)(a)"
+			"   else"
+			"     self.icons[i] = nil"
+			"   end"
 			"  end"
 			""
 			"  local call = function (id)"
@@ -888,17 +1063,26 @@ LuaInstance::init ()
 			"   collectgarbage()"
 			"  end"
 			""
-			"  local add = function (i, n, s, b, a)"
+			"  local icon = function (id, ...)"
+			"   if type(self.icons[id]) == 'function' then"
+			"     pcall (self.icons[id], ...)"
+			"   end"
+			"   collectgarbage()"
+			"  end"
+			""
+			"  local add = function (i, n, s, b, c, a)"
 			"   assert(type(b) == 'string', 'ByteCode must be string')"
-			"   load (b)()" // assigns f
+			"   f = nil load (b)()" // assigns f
+			"   icn = nil load (c)()" // may assign "icn"
 			"   assert(type(f) == 'string', 'Assigned ByteCode must be string')"
-			"   addinternal (i, n, s, load(f), a)"
+			"   addinternal (i, n, s, load(f), type(icn) ~= \"string\" or icn == '' or load(icn), a)"
 			"  end"
 			""
 			"  local get = function (id)"
 			"   if type(self.scripts[id]) == 'table' then"
 			"    return { ['name'] = self.scripts[id]['n'],"
 			"             ['script'] = self.scripts[id]['s'],"
+			"             ['icon'] = type(self.scripts[id]['c']) == 'function',"
 			"             ['args'] = self.scripts[id]['a'] }"
 			"   end"
 			"   return nil"
@@ -914,7 +1098,6 @@ LuaInstance::init ()
 			""
 			"  local function serialize (name, value)"
 			"   local rv = name .. ' = '"
-			"   collectgarbage()"
 			"   if type(value) == \"number\" or type(value) == \"string\" or type(value) == \"nil\" then"
 			"    return rv .. basic_serialize(value) .. ' '"
 			"   elseif type(value) == \"table\" then"
@@ -922,11 +1105,12 @@ LuaInstance::init ()
 			"    for k,v in pairs(value) do"
 			"     local fieldname = string.format(\"%s[%s]\", name, basic_serialize(k))"
 			"     rv = rv .. serialize(fieldname, v) .. ' '"
-			"     collectgarbage()" // string concatenation allocates a new string
 			"    end"
 			"    return rv;"
 			"   elseif type(value) == \"function\" then"
 			"     return rv .. string.format(\"%q\", string.dump(value, true))"
+			"   elseif type(value) == \"boolean\" then"
+			"     return rv .. tostring (value)"
 			"   else"
 			"    error('cannot save a ' .. type(value))"
 			"   end"
@@ -940,6 +1124,7 @@ LuaInstance::init ()
 			"  local clear = function ()"
 			"   self.scripts = {}"
 			"   self.instances = {}"
+			"   self.icons = {}"
 			"   collectgarbage()"
 			"  end"
 			""
@@ -947,19 +1132,18 @@ LuaInstance::init ()
 			"   clear()"
 			"   load (state)()"
 			"   for i, s in pairs (scripts) do"
-			"    addinternal (i, s['n'], s['s'], load(s['f']), s['a'])"
+			"    addinternal (i, s['n'], s['s'], load(s['f']), type (s['c']) ~= \"string\" or s['c'] == '' or load (s['c']), s['a'])"
 			"   end"
 			"   collectgarbage()"
 			"  end"
 			""
 			" return { call = call, add = add, remove = remove, get = get,"
-			"          restore = restore, save = save, clear = clear}"
+			"          restore = restore, save = save, clear = clear, icon = icon}"
 			" end"
 			" "
 			" manager = ScriptManager ()"
 			" ScriptManager = nil"
 			);
-
 	lua_State* L = lua.getState();
 
 	try {
@@ -971,6 +1155,7 @@ LuaInstance::init ()
 		_lua_del_action = new luabridge::LuaRef(lua_mgr["remove"]);
 		_lua_get_action = new luabridge::LuaRef(lua_mgr["get"]);
 		_lua_call_action = new luabridge::LuaRef(lua_mgr["call"]);
+		_lua_render_icon = new luabridge::LuaRef(lua_mgr["icon"]);
 		_lua_save = new luabridge::LuaRef(lua_mgr["save"]);
 		_lua_load = new luabridge::LuaRef(lua_mgr["restore"]);
 		_lua_clear = new luabridge::LuaRef(lua_mgr["clear"]);
@@ -983,6 +1168,7 @@ LuaInstance::init ()
 	}
 
 	register_classes (L);
+	register_hooks (L);
 
 	luabridge::push <PublicEditor *> (L, &PublicEditor::instance());
 	lua_setglobal (L, "Editor");
@@ -1002,6 +1188,7 @@ void LuaInstance::set_session (Session* s)
 		i->second->set_session (s);
 	}
 	point_one_second_connection = Timers::rapid_connect (sigc::mem_fun(*this, & LuaInstance::every_point_one_seconds));
+	SetSession (); /* EMIT SIGNAL */
 }
 
 void
@@ -1031,7 +1218,6 @@ LuaInstance::every_point_one_seconds ()
 int
 LuaInstance::set_state (const XMLNode& node)
 {
-	LocaleGuard lg;
 	XMLNode* child;
 
 	if ((child = find_named_node (node, "ActionScript"))) {
@@ -1074,16 +1260,25 @@ bool
 LuaInstance::interactive_add (LuaScriptInfo::ScriptType type, int id)
 {
 	std::string title;
+	std::string param_function = "action_params";
 	std::vector<std::string> reg;
 
 	switch (type) {
 		case LuaScriptInfo::EditorAction:
 			reg = lua_action_names ();
-			title = "Add Lua Action";
+			title = _("Add Lua Action");
 			break;
 		case LuaScriptInfo::EditorHook:
 			reg = lua_slot_names ();
-			title = "Add Lua Callback Hook";
+			title = _("Add Lua Callback Hook");
+			break;
+		case LuaScriptInfo::Session:
+			if (!_session) {
+				return false;
+			}
+			reg = _session->registered_lua_functions ();
+			title = _("Add Lua Session Script");
+			param_function = "sess_params";
 			break;
 		default:
 			return false;
@@ -1111,16 +1306,22 @@ LuaInstance::interactive_add (LuaScriptInfo::ScriptType type, int id)
 		return false;
 	}
 
-	LuaScriptParamList lsp = LuaScriptParams::script_params (spi, "action_params");
+	LuaScriptParamList lsp = LuaScriptParams::script_params (spi, param_function);
 
 	ScriptParameterDialog spd (_("Set Script Parameters"), spi, reg, lsp);
-	switch (spd.run ()) {
-		case Gtk::RESPONSE_ACCEPT:
-			break;
-		default:
-			return false;
+
+	if (!spd.need_interation ()) {
+		switch (spd.run ()) {
+			case Gtk::RESPONSE_ACCEPT:
+				break;
+			default:
+				return false;
+		}
 	}
 
+	LuaScriptParamPtr lspp (new LuaScriptParam("x-script-origin", "", spi->path, false));
+	lsp.push_back (lspp);
+
 	switch (type) {
 		case LuaScriptInfo::EditorAction:
 			return set_lua_action (id, spd.name(), script, lsp);
@@ -1128,6 +1329,18 @@ LuaInstance::interactive_add (LuaScriptInfo::ScriptType type, int id)
 		case LuaScriptInfo::EditorHook:
 			return register_lua_slot (spd.name(), script, lsp);
 			break;
+		case LuaScriptInfo::Session:
+			try {
+				_session->register_lua_function (spd.name(), script, lsp);
+			} catch (luabridge::LuaException const& e) {
+				string msg = string_compose (_("Session script '%1' instantiation failed: %2"), spd.name(), e.what ());
+				Gtk::MessageDialog am (msg);
+				am.run ();
+			} catch (SessionException e) {
+				string msg = string_compose (_("Loading Session script '%1' failed: %2"), spd.name(), e.what ());
+				Gtk::MessageDialog am (msg);
+				am.run ();
+			}
 		default:
 			break;
 	}
@@ -1137,7 +1350,6 @@ LuaInstance::interactive_add (LuaScriptInfo::ScriptType type, int id)
 XMLNode&
 LuaInstance::get_action_state ()
 {
-	LocaleGuard lg;
 	std::string saved;
 	{
 		luabridge::LuaRef savedstate ((*_lua_save)());
@@ -1150,7 +1362,7 @@ LuaInstance::get_action_state ()
 	g_free (b64);
 
 	XMLNode* script_node = new XMLNode (X_("ActionScript"));
-	script_node->add_property (X_("lua"), LUA_VERSION);
+	script_node->set_property (X_("lua"), LUA_VERSION);
 	script_node->add_content (b64s);
 
 	return *script_node;
@@ -1177,6 +1389,23 @@ LuaInstance::call_action (const int id)
 	}
 }
 
+void
+LuaInstance::render_action_icon (cairo_t* cr, int w, int h, uint32_t c, void* i) {
+	int ii = reinterpret_cast<uintptr_t> (i);
+	instance()->render_icon (ii, cr, w, h, c);
+}
+
+void
+LuaInstance::render_icon (int i, cairo_t* cr, int w, int h, uint32_t clr)
+{
+	 Cairo::Context ctx (cr);
+	 try {
+		 (*_lua_render_icon)(i + 1, (Cairo::Context *)&ctx, w, h, clr);
+	 } catch (luabridge::LuaException const& e) {
+		 cerr << "LuaException:" << e.what () << endl;
+	 }
+}
+
 bool
 LuaInstance::set_lua_action (
 		const int id,
@@ -1189,17 +1418,19 @@ LuaInstance::set_lua_action (
 		// get bytcode of factory-function in a sandbox
 		// (don't allow scripts to interfere)
 		const std::string& bytecode = LuaScripting::get_factory_bytecode (script);
+		const std::string& iconfunc = LuaScripting::get_factory_bytecode (script, "icon", "icn");
 		luabridge::LuaRef tbl_arg (luabridge::newTable(L));
 		for (LuaScriptParamList::const_iterator i = args.begin(); i != args.end(); ++i) {
 			if ((*i)->optional && !(*i)->is_set) { continue; }
 			tbl_arg[(*i)->name] = (*i)->value;
 		}
-		(*_lua_add_action)(id + 1, name, script, bytecode, tbl_arg);
+		(*_lua_add_action)(id + 1, name, script, bytecode, iconfunc, tbl_arg);
 		ActionChanged (id, name); /* EMIT SIGNAL */
 	} catch (luabridge::LuaException const& e) {
 		cerr << "LuaException:" << e.what () << endl;
 		return false;
 	}
+	_session->set_dirty ();
 	return true;
 }
 
@@ -1213,6 +1444,7 @@ LuaInstance::remove_lua_action (const int id)
 		return false;
 	}
 	ActionChanged (id, ""); /* EMIT SIGNAL */
+	_session->set_dirty ();
 	return true;
 }
 
@@ -1249,6 +1481,23 @@ LuaInstance::lua_action_names ()
 	return rv;
 }
 
+bool
+LuaInstance::lua_action_has_icon (const int id)
+{
+	try {
+		luabridge::LuaRef ref ((*_lua_get_action)(id + 1));
+		if (ref.isNil()) {
+			return false;
+		}
+		if (ref["icon"].isBoolean()) {
+			return ref["icon"].cast<bool>();
+		}
+	} catch (luabridge::LuaException const& e) {
+		cerr << "LuaException:" << e.what () << endl;
+	}
+	return false;
+}
+
 bool
 LuaInstance::lua_action (const int id, std::string& name, std::string& script, LuaScriptParamList& args)
 {
@@ -1293,6 +1542,7 @@ LuaInstance::register_lua_slot (const std::string& name, const std::string& scri
 	try {
 		LuaState l;
 		l.Print.connect (&_lua_print);
+		l.sandbox (true);
 		lua_State* L = l.getState();
 		register_hooks (L);
 		l.do_command ("function ardour () end");
@@ -1321,6 +1571,7 @@ LuaInstance::register_lua_slot (const std::string& name, const std::string& scri
 	} catch (luabridge::LuaException const& e) {
 		cerr << "LuaException:" << e.what () << endl;
 	}
+	_session->set_dirty ();
 	return false;
 }
 
@@ -1333,6 +1584,7 @@ LuaInstance::unregister_lua_slot (const PBD::ID& id)
 		_callbacks.erase (i);
 		return true;
 	}
+	_session->set_dirty ();
 	return false;
 }
 
@@ -1470,17 +1722,25 @@ LuaCallback::get_state (void)
 		luabridge::LuaRef savedstate ((*_lua_save)());
 		saved = savedstate.cast<std::string>();
 	}
-	lua.collect_garbage ();
+
+	lua.collect_garbage (); // this may be expensive:
+	/* Editor::instant_save() calls Editor::get_state() which
+	 * calls LuaInstance::get_hook_state() which in turn calls
+	 * this LuaCallback::get_state() for every registered hook.
+	 *
+	 * serialize in _lua_save() allocates many small strings
+	 * on the lua-stack, collecting them all may take a ms.
+	 */
 
 	gchar* b64 = g_base64_encode ((const guchar*)saved.c_str (), saved.size ());
 	std::string b64s (b64);
 	g_free (b64);
 
 	XMLNode* script_node = new XMLNode (X_("LuaCallback"));
-	script_node->add_property (X_("lua"), LUA_VERSION);
-	script_node->add_property (X_("id"), _id.to_s ());
-	script_node->add_property (X_("name"), _name);
-	script_node->add_property (X_("signals"), _signals.to_string ());
+	script_node->set_property (X_("lua"), LUA_VERSION);
+	script_node->set_property (X_("id"), _id.to_s ());
+	script_node->set_property (X_("name"), _name);
+	script_node->set_property (X_("signals"), _signals.to_string ());
 	script_node->add_content (b64s);
 	return *script_node;
 }
@@ -1489,6 +1749,7 @@ void
 LuaCallback::init (void)
 {
 	lua.Print.connect (&_lua_print);
+	lua.sandbox (false);
 
 	lua.do_command (
 			"function ScriptManager ()"
@@ -1500,7 +1761,7 @@ LuaCallback::init (void)
 			"   assert(type(f) == 'function', 'Factory is a not a function')"
 			"   assert(type(a) == 'table' or type(a) == 'nil', 'Given argument is invalid')"
 			"   self.script = { ['n'] = n, ['s'] = s, ['f'] = f, ['a'] = a }"
-			"   local env = _ENV;  env.f = nil env.debug = nil os.exit = nil require = nil dofile = nil loadfile = nil package = nil"
+			"   local env = _ENV; env.f = nil env.io = nil"
 			"   self.instance = load (string.dump(f, true), nil, nil, env)(a)"
 			"  end"
 			""
@@ -1546,7 +1807,6 @@ LuaCallback::init (void)
 			""
 			"  local function serialize (name, value)"
 			"   local rv = name .. ' = '"
-			"   collectgarbage()"
 			"   if type(value) == \"number\" or type(value) == \"string\" or type(value) == \"nil\" then"
 			"    return rv .. basic_serialize(value) .. ' '"
 			"   elseif type(value) == \"table\" then"
@@ -1554,11 +1814,12 @@ LuaCallback::init (void)
 			"    for k,v in pairs(value) do"
 			"     local fieldname = string.format(\"%s[%s]\", name, basic_serialize(k))"
 			"     rv = rv .. serialize(fieldname, v) .. ' '"
-			"     collectgarbage()" // string concatenation allocates a new string
 			"    end"
 			"    return rv;"
 			"   elseif type(value) == \"function\" then"
 			"     return rv .. string.format(\"%q\", string.dump(value, true))"
+			"   elseif type(value) == \"boolean\" then"
+			"     return rv .. tostring (value)"
 			"   else"
 			"    error('cannot save a ' .. type(value))"
 			"   end"
@@ -1605,6 +1866,7 @@ LuaCallback::init (void)
 	}
 
 	LuaInstance::register_classes (L);
+	LuaInstance::register_hooks (L);
 
 	luabridge::push <PublicEditor *> (L, &PublicEditor::instance());
 	lua_setglobal (L, "Editor");
@@ -1674,10 +1936,6 @@ LuaCallback::session_going_away ()
 	_session = 0;
 
 	drop_callback (); /* EMIT SIGNAL */
-
-	lua_State* L = lua.getState();
-	LuaBindings::set_session (L, 0);
-	lua.do_command ("collectgarbage();");
 }
 
 void
