@@ -34,10 +34,12 @@
 #include "pbd/md5.h"
 
 #include "gtkmm2ext/gtk_ui.h"
-#include "gtkmm2ext/pane.h"
 #include "gtkmm2ext/utils.h"
 #include "gtkmm2ext/window_title.h"
 
+#include "widgets/pane.h"
+#include "widgets/tooltips.h"
+
 #include "ardour/filesystem_paths.h"
 #include "ardour/luabindings.h"
 #include "LuaBridge/LuaBridge.h"
@@ -47,14 +49,12 @@
 #include "luainstance.h"
 #include "luawindow.h"
 #include "public_editor.h"
-#include "tooltips.h"
 #include "utils.h"
 #include "utils_videotl.h"
 
 #include "pbd/i18n.h"
 
 using namespace ARDOUR;
-using namespace ARDOUR_UI_UTILS;
 using namespace PBD;
 using namespace Gtk;
 using namespace Glib;
@@ -156,14 +156,15 @@ LuaWindow::LuaWindow ()
 	vbox->pack_start (*scrollin, true, true, 0);
 	vbox->pack_start (*hbox, false, false, 2);
 
-	Gtkmm2ext::VPane *vpane = manage (new Gtkmm2ext::VPane ());
+	ArdourWidgets::VPane *vpane = manage (new ArdourWidgets::VPane ());
 	vpane->add (*vbox);
 	vpane->add (scrollout);
+	vpane->set_divider (0, 0.75);
 
 	vpane->show_all ();
 	add (*vpane);
 	set_size_request (640, 480); // XXX
-	ARDOUR_UI_UTILS::set_tooltip (script_select, _("Select Editor Buffer"));
+	ArdourWidgets::set_tooltip (script_select, _("Select Editor Buffer"));
 
 	setup_buffers ();
 	LuaScripting::instance().scripts_changed.connect (*this, invalidator (*this), boost::bind (&LuaWindow::refresh_scriptlist, this), gui_context());
@@ -189,7 +190,7 @@ LuaWindow::hide_window (GdkEventAny *ev)
 {
 	if (!_visible) return 0;
 	_visible = false;
-	return just_hide_it (ev, static_cast<Gtk::Window *>(this));
+	return ARDOUR_UI_UTILS::just_hide_it (ev, static_cast<Gtk::Window *>(this));
 }
 
 void LuaWindow::reinit_lua ()
@@ -198,6 +199,7 @@ void LuaWindow::reinit_lua ()
 	delete lua;
 	lua = new LuaState();
 	lua->Print.connect (sigc::mem_fun (*this, &LuaWindow::append_text));
+	lua->sandbox (false);
 
 	lua_State* L = lua->getState();
 	LuaInstance::register_classes (L);
@@ -322,6 +324,17 @@ LuaWindow::clear_output ()
 	tb->set_text ("");
 }
 
+void
+LuaWindow::edit_script (const std::string& name, const std::string& script)
+{
+	ScriptBuffer* sb = new LuaWindow::ScriptBuffer (name);
+	sb->script = script;
+	script_buffers.push_back (ScriptBufferPtr (sb));
+	script_selection_changed (script_buffers.back ());
+	refresh_scriptlist ();
+	show_window ();
+}
+
 void
 LuaWindow::new_script ()
 {
