@@ -20,13 +20,7 @@
 #include <map>
 #include <boost/algorithm/string.hpp>
 
-#include <gtkmm2ext/gtk_ui.h>
-#include <gtkmm2ext/choice.h>
-#include <gtkmm2ext/doi.h>
-#include <gtkmm2ext/bindable_button.h>
-#include <gtkmm2ext/barcontroller.h>
-#include <gtkmm2ext/gtk_ui.h>
-#include <gtkmm2ext/utils.h>
+#include <gtkmm/stock.h>
 
 #include "pbd/memento_command.h"
 #include "pbd/stacktrace.h"
@@ -53,7 +47,15 @@
 #include "ardour/session.h"
 #include "ardour/template_utils.h"
 
-#include "ardour_button.h"
+#include "gtkmm2ext/gtk_ui.h"
+#include "gtkmm2ext/doi.h"
+#include "gtkmm2ext/gtk_ui.h"
+#include "gtkmm2ext/utils.h"
+
+#include "widgets/ardour_button.h"
+#include "widgets/binding_proxy.h"
+#include "widgets/prompter.h"
+
 #include "ardour_dialog.h"
 #include "ardour_ui.h"
 #include "automation_time_axis.h"
@@ -65,7 +67,6 @@
 #include "latency_gui.h"
 #include "mixer_strip.h"
 #include "plugin_pin_dialog.h"
-#include "prompter.h"
 #include "rgb_macros.h"
 #include "route_time_axis.h"
 #include "route_ui.h"
@@ -73,12 +74,13 @@
 #include "ui_config.h"
 #include "utils.h"
 
-
 #include "pbd/i18n.h"
+
 using namespace Gtk;
 using namespace Gtkmm2ext;
 using namespace ARDOUR;
 using namespace ARDOUR_UI_UTILS;
+using namespace ArdourWidgets;
 using namespace PBD;
 using namespace std;
 
@@ -243,6 +245,8 @@ RouteUI::reset ()
 	delete mute_menu;
 	mute_menu = 0;
 
+	_color_picker.reset ();
+
 	denormal_menu_item = 0;
 }
 
@@ -259,6 +263,41 @@ RouteUI::set_route (boost::shared_ptr<Route> rp)
 
 	_route = rp;
 
+	if ( !_route->presentation_info().color_set() ) {
+		/* deal with older 4.x color, which was stored in the GUI object state */
+
+		string p = ARDOUR_UI::instance()->gui_object_state->get_string (route_state_id(), X_("color"));
+
+		if (!p.empty()) {
+
+			/* old v4.x or earlier session. Use this information */
+
+			int red, green, blue;
+			char colon;
+
+			stringstream ss (p);
+
+			/* old color format version was:
+
+			   16bit value for red:16 bit value for green:16 bit value for blue
+
+			   decode to rgb ..
+			*/
+
+			ss >> red;
+			ss >> colon;
+			ss >> green;
+			ss >> colon;
+			ss >> blue;
+
+			red >>= 2;
+			green >>= 2;
+			blue >>= 2;
+
+			_route->presentation_info().set_color (RGBA_TO_UINT (red, green, blue, 255));
+		}
+	}
+
 	if (set_color_from_route()) {
 		set_color (gdk_color_to_rgba (AxisView::unique_random_color ()));
 	}
@@ -285,10 +324,13 @@ RouteUI::set_route (boost::shared_ptr<Route> rp)
 	_route->solo_safe_control()->Changed.connect (route_connections, invalidator (*this), boost::bind (&RouteUI::update_solo_display, this), gui_context());
 	_route->solo_isolate_control()->Changed.connect (route_connections, invalidator (*this), boost::bind (&RouteUI::update_solo_display, this), gui_context());
 	_route->phase_control()->Changed.connect (route_connections, invalidator (*this), boost::bind (&RouteUI::polarity_changed, this), gui_context());
+	_route->fan_out.connect (route_connections, invalidator (*this), boost::bind (&RouteUI::fan_out, this, true, true), gui_context());
 
 	if (is_track()) {
 		track()->FreezeChange.connect (*this, invalidator (*this), boost::bind (&RouteUI::map_frozen, this), gui_context());
+#ifdef XXX_OLD_DESTRUCTIVE_API_XXX
 		track()->TrackModeChanged.connect (route_connections, invalidator (*this), boost::bind (&RouteUI::track_mode_changed, this), gui_context());
+#endif
 		track_mode_changed();
 	}
 
@@ -307,10 +349,10 @@ RouteUI::set_route (boost::shared_ptr<Route> rp)
 		rec_enable_button->show();
 		rec_enable_button->set_controllable (t->rec_enable_control());
 
-                if (is_midi_track()) {
-                        midi_track()->StepEditStatusChange.connect (route_connections, invalidator (*this),
-                                                                    boost::bind (&RouteUI::step_edit_changed, this, _1), gui_context());
-                }
+		if (is_midi_track()) {
+			midi_track()->StepEditStatusChange.connect (route_connections, invalidator (*this),
+					boost::bind (&RouteUI::step_edit_changed, this, _1), gui_context());
+		}
 
 	}
 
@@ -360,9 +402,9 @@ RouteUI::set_route (boost::shared_ptr<Route> rp)
 void
 RouteUI::polarity_changed ()
 {
-        if (!_route) {
-                return;
-        }
+	if (!_route) {
+		return;
+	}
 
 	set_invert_button_state ();
 }
@@ -375,7 +417,7 @@ RouteUI::mute_press (GdkEventButton* ev)
 	}
 
 	//if this is a binding action, let the ArdourButton handle it
-	if ( BindingProxy::is_bind_action(ev) )
+	if (BindingProxy::is_bind_action(ev) )
 		return false;
 
 	multiple_mute_change = false;
@@ -454,6 +496,8 @@ RouteUI::mute_press (GdkEventButton* ev)
 						_mute_release->routes = rl;
 					}
 
+					boost::shared_ptr<MuteControl> mc = _route->mute_control();
+					mc->start_touch (_session->audible_frame ());
 					_session->set_controls (route_list_to_control_list (rl, &Stripable::mute_control), _route->muted_by_self() ? 0.0 : 1.0, Controllable::InverseGroup);
 				}
 
@@ -468,7 +512,9 @@ RouteUI::mute_press (GdkEventButton* ev)
 					_mute_release->routes = rl;
 				}
 
-				_route->mute_control()->set_value (!_route->muted_by_self(), Controllable::UseGroup);
+				boost::shared_ptr<MuteControl> mc = _route->mute_control();
+				mc->start_touch (_session->audible_frame ());
+				mc->set_value (!_route->muted_by_self(), Controllable::UseGroup);
 			}
 		}
 	}
@@ -485,6 +531,8 @@ RouteUI::mute_release (GdkEventButton* /*ev*/)
 		_mute_release = 0;
 	}
 
+	_route->mute_control()->stop_touch (_session->audible_frame ());
+
 	return false;
 }
 
@@ -544,7 +592,7 @@ RouteUI::solo_press(GdkEventButton* ev)
 	}
 
 	//if this is a binding action, let the ArdourButton handle it
-	if ( BindingProxy::is_bind_action(ev) )
+	if (BindingProxy::is_bind_action(ev) )
 		return false;
 
 	multiple_solo_change = false;
@@ -583,14 +631,21 @@ RouteUI::solo_press(GdkEventButton* ev)
 
 			} else if (Keyboard::modifier_state_contains (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier|Keyboard::SecondaryModifier))) {
 
-				// Primary-Secondary-click: exclusively solo this track
+				/* Primary-Secondary-click: exclusively solo this track */
 
 				if (_solo_release) {
 					_solo_release->exclusive = true;
 
-					boost::shared_ptr<RouteList> routes = _session->get_routes();
+					_solo_release->routes_on.reset (new RouteList);
+					_solo_release->routes_off.reset (new RouteList);
 
-					for (RouteList::iterator i = routes->begin(); i != routes->end(); ++i) {
+					boost::shared_ptr<RouteList> routes = _session->get_routes();
+					for (RouteList::const_iterator i = routes->begin(); i != routes->end(); ++i) {
+#ifdef MIXBUS
+						if ((0 == _route->mixbus()) != (0 == (*i)->mixbus ())) {
+							continue;
+						}
+#endif
 						if ((*i)->soloed ()) {
 							_solo_release->routes_on->push_back (*i);
 						} else {
@@ -599,6 +654,20 @@ RouteUI::solo_press(GdkEventButton* ev)
 					}
 				}
 
+				boost::shared_ptr<RouteList> rl (new RouteList);
+				boost::shared_ptr<RouteList> routes = _session->get_routes();
+				for (RouteList::const_iterator i = routes->begin(); i != routes->end(); ++i) {
+#ifdef MIXBUS
+					if ((0 == _route->mixbus()) != (0 == (*i)->mixbus ())) {
+						continue;
+					}
+#endif
+					if ((*i)->soloed ()) {
+						rl->push_back (*i);
+					}
+				}
+				_session->set_controls (route_list_to_control_list (rl, &Stripable::solo_control), false, Controllable::UseGroup);
+
 				if (Config->get_solo_control_is_listen_control()) {
 					/* ??? we need a just_one_listen() method */
 				} else {
@@ -677,9 +746,9 @@ bool
 RouteUI::solo_release (GdkEventButton* /*ev*/)
 {
 	if (_solo_release) {
-
 		if (_solo_release->exclusive) {
-
+			_session->set_controls (route_list_to_control_list (_solo_release->routes_off, &Stripable::solo_control), 0.0, Controllable::NoGroup);
+			_session->set_controls (route_list_to_control_list (_solo_release->routes_on, &Stripable::solo_control), 1.0, Controllable::NoGroup);
 		} else {
 			_session->set_controls (route_list_to_control_list (_solo_release->routes, &Stripable::solo_control), _solo_release->active ? 1.0 : 0.0, Controllable::UseGroup);
 		}
@@ -699,11 +768,11 @@ RouteUI::rec_enable_press(GdkEventButton* ev)
 	}
 
 	//if this is a binding action, let the ArdourButton handle it
-	if ( BindingProxy::is_bind_action(ev) )
+	if (BindingProxy::is_bind_action(ev) )
 		return false;
 
 	if (!_session->engine().connected()) {
-	        MessageDialog msg (_("Not connected to AudioEngine - cannot engage record"));
+		MessageDialog msg (_("Not connected to AudioEngine - cannot engage record"));
 		msg.run ();
 		return false;
 	}
@@ -849,22 +918,20 @@ RouteUI::monitor_release (GdkEventButton* ev, MonitorChoice monitor_choice)
 	}
 
 	if (Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier|Keyboard::TertiaryModifier))) {
+		/* Primary-Tertiary-click applies change to all routes */
 		rl = _session->get_routes ();
-
+		_session->set_controls (route_list_to_control_list (rl, &Stripable::monitoring_control), (double) mc, Controllable::NoGroup);
 	} else if (Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
-		if (_route->route_group() && _route->route_group()->is_monitoring()) {
-			rl = _route->route_group()->route_list();
-		} else {
-			rl.reset (new RouteList);
-			rl->push_back (route());
-		}
+		/* Primary-click overrides group */
+		rl.reset (new RouteList);
+		rl->push_back (route());
+		_session->set_controls (route_list_to_control_list (rl, &Stripable::monitoring_control), (double) mc, Controllable::InverseGroup);
 	} else {
 		rl.reset (new RouteList);
 		rl->push_back (route());
+		_session->set_controls (route_list_to_control_list (rl, &Stripable::monitoring_control), (double) mc, Controllable::UseGroup);
 	}
 
-	_session->set_controls (route_list_to_control_list (rl, &Stripable::monitoring_control), (double) mc, Controllable::UseGroup);
-
 	return false;
 }
 
@@ -1197,28 +1264,28 @@ RouteUI::update_solo_display ()
 		solo_isolated_check->set_active (yn);
 	}
 
-        set_button_names ();
+	set_button_names ();
 
-        if (solo_isolated_led) {
-	        if (_route->solo_isolate_control()->solo_isolated()) {
+	if (solo_isolated_led) {
+		if (_route->solo_isolate_control()->solo_isolated()) {
 			solo_isolated_led->set_active_state (Gtkmm2ext::ExplicitActive);
 		} else {
 			solo_isolated_led->unset_active_state ();
 		}
-        }
+	}
 
-        if (solo_safe_led) {
-	        if (_route->solo_safe_control()->solo_safe()) {
+	if (solo_safe_led) {
+		if (_route->solo_safe_control()->solo_safe()) {
 			solo_safe_led->set_active_state (Gtkmm2ext::ExplicitActive);
 		} else {
 			solo_safe_led->unset_active_state ();
 		}
-        }
+	}
 
 	solo_button->set_active_state (solo_active_state (_route));
 
-        /* some changes to solo status can affect mute display, so catch up
-         */
+	/* some changes to solo status can affect mute display, so catch up
+	 */
 
 	update_mute_display ();
 }
@@ -1275,11 +1342,11 @@ RouteUI::mute_active_state (Session*, boost::shared_ptr<Stripable> s)
 void
 RouteUI::update_mute_display ()
 {
-        if (!_route) {
-                return;
-        }
+	if (!_route) {
+		return;
+	}
 
-        mute_button->set_active_state (mute_active_state (_session, _route));
+	mute_button->set_active_state (mute_active_state (_session, _route));
 }
 
 
@@ -1311,31 +1378,31 @@ RouteUI::blink_rec_display (bool blinkOn)
 	}
 
 	if (track()->rec_enable_control()->get_value()) {
-                switch (_session->record_status ()) {
-                case Session::Recording:
-                        rec_enable_button->set_active_state (Gtkmm2ext::ExplicitActive);
-                        break;
+		switch (_session->record_status ()) {
+			case Session::Recording:
+				rec_enable_button->set_active_state (Gtkmm2ext::ExplicitActive);
+				break;
 
-                case Session::Disabled:
-                case Session::Enabled:
-	                if (UIConfiguration::instance().get_blink_rec_arm()) {
-	                        rec_enable_button->set_active_state ( blinkOn ? Gtkmm2ext::ExplicitActive : Gtkmm2ext::Off );
-	                } else {
-	                        rec_enable_button->set_active_state ( ImplicitActive );
-	                }
-                        break;
-                }
+			case Session::Disabled:
+			case Session::Enabled:
+				if (UIConfiguration::instance().get_blink_rec_arm()) {
+					rec_enable_button->set_active_state ( blinkOn ? Gtkmm2ext::ExplicitActive : Gtkmm2ext::Off );
+				} else {
+					rec_enable_button->set_active_state ( ImplicitActive );
+				}
+				break;
+		}
 
-                if (step_edit_item) {
-                        step_edit_item->set_sensitive (false);
-                }
+		if (step_edit_item) {
+			step_edit_item->set_sensitive (false);
+		}
 
 	} else {
 		rec_enable_button->unset_active_state ();
 
-                if (step_edit_item) {
-                        step_edit_item->set_sensitive (true);
-                }
+		if (step_edit_item) {
+			step_edit_item->set_sensitive (true);
+		}
 	}
 
 	check_rec_enable_sensitivity ();
@@ -1355,19 +1422,15 @@ RouteUI::build_solo_menu (void)
 	check->set_active (_route->solo_isolate_control()->solo_isolated());
 	check->signal_toggled().connect (sigc::bind (sigc::mem_fun (*this, &RouteUI::toggle_solo_isolated), check));
 	items.push_back (CheckMenuElem(*check));
-        solo_isolated_check = dynamic_cast<Gtk::CheckMenuItem*>(&items.back());
+	solo_isolated_check = dynamic_cast<Gtk::CheckMenuItem*>(&items.back());
 	check->show_all();
 
 	check = new Gtk::CheckMenuItem(_("Solo Safe"));
 	check->set_active (_route->solo_safe_control()->solo_safe());
 	check->signal_toggled().connect (sigc::bind (sigc::mem_fun (*this, &RouteUI::toggle_solo_safe), check));
 	items.push_back (CheckMenuElem(*check));
-        solo_safe_check = dynamic_cast<Gtk::CheckMenuItem*>(&items.back());
+	solo_safe_check = dynamic_cast<Gtk::CheckMenuItem*>(&items.back());
 	check->show_all();
-
-	//items.push_back (SeparatorElem());
-	// items.push_back (MenuElem (_("MIDI Bind"), sigc::mem_fun (*mute_button, &BindableToggleButton::midi_learn)));
-
 }
 
 void
@@ -1404,9 +1467,6 @@ RouteUI::build_mute_menu(void)
 	items.push_back (CheckMenuElem(*main_mute_check));
 	main_mute_check->show_all();
 
-	//items.push_back (SeparatorElem());
-	// items.push_back (MenuElem (_("MIDI Bind"), sigc::mem_fun (*mute_button, &BindableToggleButton::midi_learn)));
-
 	_route->mute_points_changed.connect (route_connections, invalidator (*this), boost::bind (&RouteUI::muting_change, this), gui_context());
 }
 
@@ -1539,14 +1599,14 @@ RouteUI::solo_safe_button_release (GdkEventButton* ev)
 void
 RouteUI::toggle_solo_isolated (Gtk::CheckMenuItem* check)
 {
-        bool view = check->get_active();
-        bool model = _route->solo_isolate_control()->solo_isolated();
+	bool view = check->get_active();
+	bool model = _route->solo_isolate_control()->solo_isolated();
 
-        /* called AFTER the view has changed */
+	/* called AFTER the view has changed */
 
-        if (model != view) {
-	        _route->solo_isolate_control()->set_value (view ? 1.0 : 0.0, Controllable::UseGroup);
-        }
+	if (model != view) {
+		_route->solo_isolate_control()->set_value (view ? 1.0 : 0.0, Controllable::UseGroup);
+	}
 }
 
 void
@@ -1555,18 +1615,11 @@ RouteUI::toggle_solo_safe (Gtk::CheckMenuItem* check)
 	_route->solo_safe_control()->set_value (check->get_active() ? 1.0 : 0.0, Controllable::UseGroup);
 }
 
-/** Ask the user to choose a colour, and then apply that color to my route
- */
+/** Ask the user to choose a colour, and then apply that color to my route */
 void
 RouteUI::choose_color ()
 {
-	bool picked;
-	Gdk::Color c (gdk_color_from_rgba (_route->presentation_info().color()));
-	Gdk::Color const color = Gtkmm2ext::UI::instance()->get_color (_("Color Selection"), picked, &c);
-
-	if (picked) {
-		set_color (gdk_color_to_rgba (color));
-	}
+	_color_picker.popup (_route);
 }
 
 /** Set the route's own color.  This may not be used for display if
@@ -1617,7 +1670,7 @@ RouteUI::verify_new_route_name (const std::string& name)
 void
 RouteUI::route_rename ()
 {
-	ArdourPrompter name_prompter (true);
+	ArdourWidgets::Prompter name_prompter (true);
 	string result;
 	bool done = false;
 
@@ -1843,7 +1896,7 @@ RouteUI::adjust_latency ()
 }
 
 bool
-RouteUI::process_save_template_prompter (ArdourPrompter& prompter, const std::string& dir)
+RouteUI::process_save_template_prompter (Prompter& prompter, const std::string& dir)
 {
 	std::string path;
 	std::string safe_name;
@@ -1883,7 +1936,7 @@ RouteUI::save_as_template ()
 		return;
 	}
 
-	ArdourPrompter prompter (true); // modal
+	Prompter prompter (true); // modal
 
 	prompter.set_title (_("Save As Template"));
 	prompter.set_prompt (_("Template name:"));
@@ -1938,6 +1991,8 @@ RouteUI::parameter_changed (string const & p)
 		check_rec_enable_sensitivity ();
 	} else if (p == "use-monitor-bus" || p == "solo-control-is-listen-control" || p == "listen-position") {
 		set_button_names ();
+	} else if (p == "session-monitoring") {
+		update_monitoring_display ();
 	} else if (p == "auto-input") {
 		update_monitoring_display ();
 	} else if (p == "blink-rec-arm") {
@@ -1951,30 +2006,6 @@ RouteUI::parameter_changed (string const & p)
 	}
 }
 
-void
-RouteUI::step_gain_up ()
-{
-	_route->gain_control()->set_value (dB_to_coefficient (accurate_coefficient_to_dB (_route->gain_control()->get_value()) + 0.1), Controllable::UseGroup);
-}
-
-void
-RouteUI::page_gain_up ()
-{
-	_route->gain_control()->set_value (dB_to_coefficient (accurate_coefficient_to_dB (_route->gain_control()->get_value()) + 0.5), Controllable::UseGroup);
-}
-
-void
-RouteUI::step_gain_down ()
-{
-	_route->gain_control()->set_value (dB_to_coefficient (accurate_coefficient_to_dB (_route->gain_control()->get_value()) - 0.1), Controllable::UseGroup);
-}
-
-void
-RouteUI::page_gain_down ()
-{
-	_route->gain_control()->set_value (dB_to_coefficient (accurate_coefficient_to_dB (_route->gain_control()->get_value()) - 0.5), Controllable::UseGroup);
-}
-
 void
 RouteUI::setup_invert_buttons ()
 {
@@ -2116,16 +2147,8 @@ RouteUI::invert_menu_toggled (uint32_t c)
 void
 RouteUI::set_invert_sensitive (bool yn)
 {
-        for (vector<ArdourButton*>::iterator b = _invert_buttons.begin(); b != _invert_buttons.end(); ++b) {
-                (*b)->set_sensitive (yn);
-        }
-}
-
-void
-RouteUI::request_redraw ()
-{
-	if (_route) {
-		_route->gui_changed ("track_height", (void *) 0); /* EMIT_SIGNAL */
+	for (vector<ArdourButton*>::iterator b = _invert_buttons.begin(); b != _invert_buttons.end(); ++b) {
+		(*b)->set_sensitive (yn);
 	}
 }
 
@@ -2157,9 +2180,8 @@ RouteUI::track_mode_changed (void)
 }
 
 /** @return the color that this route should use; it maybe its own,
-    or it maybe that of its route group.
-*/
-
+ *  or it maybe that of its route group.
+ */
 Gdk::Color
 RouteUI::route_color () const
 {
@@ -2170,40 +2192,6 @@ RouteUI::route_color () const
 	if (g && g->is_color()) {
 		set_color_from_rgba (c, GroupTabs::group_color (g));
 	} else {
-
-		/* deal with older 4.x color, which was stored in the GUI object state */
-
-		string p = ARDOUR_UI::instance()->gui_object_state->get_string (route_state_id(), X_("color"));
-
-		if (!p.empty()) {
-
-			/* old v4.x or earlier session. Use this information */
-
-			int red, green, blue;
-			char colon;
-
-			stringstream ss (p);
-
-			/* old color format version was:
-
-			   16bit value for red:16 bit value for green:16 bit value for blue
-
-			   decode to rgb ..
-			*/
-
-			ss >> red;
-			ss >> colon;
-			ss >> green;
-			ss >> colon;
-			ss >> blue;
-
-			red >>= 2;
-			green >>= 2;
-			blue >>= 2;
-
-			_route->presentation_info().set_color (RGBA_TO_UINT (red, green, blue, 255));
-		}
-
 		set_color_from_rgba (c, _route->presentation_info().color());
 	}
 
@@ -2285,6 +2273,7 @@ RoutePinWindowProxy::route_going_away ()
 	_window = 0;
 	WM::Manager::instance().remove (this);
 	going_away_connection.disconnect();
+	delete this;
 }
 
 void
