@@ -30,6 +30,7 @@
 #include <glibmm/datetime.h>
 
 #include <gtkmm/filechooser.h>
+#include <gtkmm/stock.h>
 
 #include "pbd/basename.h"
 #include "pbd/failed_constructor.h"
@@ -43,8 +44,11 @@
 #include "gtkmm2ext/utils.h"
 #include "gtkmm2ext/keyboard.h"
 
+#include "widgets/tooltips.h"
+
 #include "ardour/audioengine.h"
 #include "ardour/filesystem_paths.h"
+#include "ardour/luascripting.h"
 #include "ardour/recent_sessions.h"
 #include "ardour/session.h"
 #include "ardour/session_state_utils.h"
@@ -56,7 +60,6 @@
 #include "opts.h"
 #include "engine_dialog.h"
 #include "pbd/i18n.h"
-#include "tooltips.h"
 #include "ui_config.h"
 #include "utils.h"
 
@@ -66,6 +69,7 @@ using namespace Gdk;
 using namespace Glib;
 using namespace PBD;
 using namespace ARDOUR;
+using namespace ArdourWidgets;
 using namespace ARDOUR_UI_UTILS;
 
 SessionDialog::SessionDialog (bool require_new, const std::string& session_name, const std::string& session_path, const std::string& template_name, bool cancel_not_quit)
@@ -412,6 +416,12 @@ SessionDialog::setup_initial_choice_box ()
 	session_filter.add_pattern (string_compose(X_("*%1"), ARDOUR::statefile_suffix));
 	session_filter.set_name (string_compose (_("%1 sessions"), PROGRAM_NAME));
 	existing_session_chooser.add_filter (session_filter);
+
+	FileFilter archive_filter;
+	archive_filter.add_pattern (X_("*.tar.xz"));
+	archive_filter.set_name (_("Session Archives"));
+	existing_session_chooser.add_filter (archive_filter);
+
 	existing_session_chooser.set_filter (session_filter);
 
 	Gtkmm2ext::add_volume_shortcuts (existing_session_chooser);
@@ -507,6 +517,16 @@ SessionDialog::populate_session_templates ()
 
 		row[session_template_columns.name] = (*x).name;
 		row[session_template_columns.path] = (*x).path;
+		row[session_template_columns.desc] = (*x).description;
+	}
+
+	LuaScriptList& ms (LuaScripting::instance ().scripts (LuaScriptInfo::SessionSetup));
+	for (LuaScriptList::const_iterator s = ms.begin(); s != ms.end(); ++s) {
+		TreeModel::Row row;
+		row = *(template_model->append ());
+		row[session_template_columns.name] = "Meta: " + (*s)->name;
+		row[session_template_columns.path] = "urn:ardour:" + (*s)->path;
+		row[session_template_columns.desc] = "urn:ardour:" + (*s)->description;
 	}
 
 	if (!templates.empty()) {
@@ -602,6 +622,7 @@ SessionDialog::setup_new_session_page ()
 
 	HBox* hbox4a = manage (new HBox);
 	use_template_button.set_label (_("Use this template"));
+	use_template_button.signal_toggled().connect(sigc::mem_fun (*this, &SessionDialog::template_checkbox_toggled));
 
 	TreeModel::Row row = *template_model->prepend ();
 	row[session_template_columns.name] = (_("no template"));
@@ -640,6 +661,14 @@ SessionDialog::setup_new_session_page ()
 
 	session_new_vbox.pack_start (*vbox2, false, false);
 	session_new_vbox.show_all ();
+
+	template_checkbox_toggled ();
+}
+
+void
+SessionDialog::template_checkbox_toggled ()
+{
+	template_chooser.set_sensitive (use_template_button.get_active());
 }
 
 void
@@ -728,6 +757,7 @@ SessionDialog::redisplay_recent_sessions ()
 
 		float sr;
 		SampleFormat sf;
+		std::string program_version;
 
 		std::string state_file_basename;
 
@@ -749,10 +779,9 @@ SessionDialog::redisplay_recent_sessions ()
 		g_stat (s.c_str(), &gsb);
 
 		row[recent_session_columns.fullpath] = s;
-		row[recent_session_columns.tip] = Gtkmm2ext::markup_escape_text (dirname);
 		row[recent_session_columns.time_modified] = gsb.st_mtime;
 
-		if (Session::get_info_from_path (s, sr, sf) == 0) {
+		if (Session::get_info_from_path (s, sr, sf, program_version) == 0) {
 			row[recent_session_columns.sample_rate] = rate_as_string (sr);
 			switch (sf) {
 			case FormatFloat:
@@ -770,6 +799,12 @@ SessionDialog::redisplay_recent_sessions ()
 			row[recent_session_columns.disk_format] = "--";
 		}
 
+		if (program_version.empty()) {
+			row[recent_session_columns.tip] = Gtkmm2ext::markup_escape_text (dirname);
+		} else {
+			row[recent_session_columns.tip] = Gtkmm2ext::markup_escape_text (dirname + "\n" + string_compose (_("Last modified with: %1"), program_version));
+		}
+
 		++session_snapshot_count;
 
 		if (state_file_names.size() > 1) {
@@ -781,6 +816,7 @@ SessionDialog::redisplay_recent_sessions ()
 			int64_t most_recent = 0;
 
 			// add the children
+			int kidcount = 0;
 			for (std::vector<std::string>::iterator i2 = state_file_names.begin(); i2 != state_file_names.end(); ++i2) {
 
 				s = Glib::build_filename (dirname, *i2 + statefile_suffix);
@@ -799,22 +835,36 @@ SessionDialog::redisplay_recent_sessions ()
 					most_recent = gsb.st_mtime;
 				}
 
-				if (Session::get_info_from_path (s, sr, sf) == 0) {
-					child_row[recent_session_columns.sample_rate] = rate_as_string (sr);
-					switch (sf) {
-					case FormatFloat:
-						child_row[recent_session_columns.disk_format] = _("32-bit float");
-						break;
-					case FormatInt24:
-						child_row[recent_session_columns.disk_format] = _("24-bit");
-						break;
-					case FormatInt16:
-						child_row[recent_session_columns.disk_format] = _("16-bit");
-						break;
+				if (++kidcount < 5) {
+					// parse "modified with" for the first 5 snapshots
+					if (Session::get_info_from_path (s, sr, sf, program_version) == 0) {
+#if 0
+						child_row[recent_session_columns.sample_rate] = rate_as_string (sr);
+						switch (sf) {
+							case FormatFloat:
+								child_row[recent_session_columns.disk_format] = _("32-bit float");
+								break;
+							case FormatInt24:
+								child_row[recent_session_columns.disk_format] = _("24-bit");
+								break;
+							case FormatInt16:
+								child_row[recent_session_columns.disk_format] = _("16-bit");
+								break;
+						}
+#else
+						child_row[recent_session_columns.sample_rate] = "";
+						child_row[recent_session_columns.disk_format] = "";
+#endif
+					} else {
+						child_row[recent_session_columns.sample_rate] = "??";
+						child_row[recent_session_columns.disk_format] = "--";
+					}
+					if (!program_version.empty()) {
+						child_row[recent_session_columns.tip] = Gtkmm2ext::markup_escape_text (string_compose (_("Last modified with: %1"), program_version));
 					}
 				} else {
-					child_row[recent_session_columns.sample_rate] = "??";
-					child_row[recent_session_columns.disk_format] = "--";
+					child_row[recent_session_columns.sample_rate] = "";
+					child_row[recent_session_columns.disk_format] = "";
 				}
 
 				++session_snapshot_count;
@@ -1217,11 +1267,16 @@ SessionDialog::recent_context_mennu (GdkEventButton *ev)
 		return;
 	}
 
+	Gtk::TreeModel::Path tpath = recent_session_model->get_path(iter);
+	const bool is_child = tpath.up () && tpath.up ();
+
 	Gtk::Menu* m = manage (new Menu);
 	MenuList& items = m->items ();
-	items.push_back (MenuElem (s));
-	items.push_back (SeparatorElem());
-	items.push_back (MenuElem (_("Remove from recent"), sigc::mem_fun (*this, &SessionDialog::recent_remove_selected)));
+	items.push_back (MenuElem (s, sigc::bind (sigc::hide_return (sigc::ptr_fun (&PBD::open_folder)), s)));
+	if (!is_child) {
+		items.push_back (SeparatorElem());
+		items.push_back (MenuElem (_("Remove session from recent list"), sigc::mem_fun (*this, &SessionDialog::recent_remove_selected)));
+	}
 	m->popup (ev->button, ev->time);
 }
 
