@@ -34,7 +34,7 @@ SessionOptionEditor::SessionOptionEditor (Session* s)
 {
 	set_session (s);
 
-        set_name ("SessionProperties");
+	set_name ("SessionProperties");
 
 	/* TIMECODE*/
 
@@ -78,21 +78,6 @@ SessionOptionEditor::SessionOptionEditor (Session* s)
 	_vpu->add (-4.1667 - 0.1, _("-4.1667 - 0.1%"));
 
 	add_option (_("Timecode"), _vpu);
-
-	add_option (_("Sync"), new BoolOption (
-			    "use-video-file-fps",
-			    _("Use Video File's FPS Instead of Timecode Value for Timeline and Video Monitor."),
-			    sigc::mem_fun (*_session_config, &SessionConfiguration::get_use_video_file_fps),
-			    sigc::mem_fun (*_session_config, &SessionConfiguration::set_use_video_file_fps)
-			    ));
-
-	add_option (_("Sync"), new BoolOption (
-			    "videotimeline-pullup",
-			    _("Apply Pull-Up/Down to Video Timeline and Video Monitor (Unless using JACK-sync)."),
-			    sigc::mem_fun (*_session_config, &SessionConfiguration::get_videotimeline_pullup),
-			    sigc::mem_fun (*_session_config, &SessionConfiguration::set_videotimeline_pullup)
-			    ));
-
 	add_option (_("Timecode"), new OptionEditorHeading (_("Ext Timecode Offsets")));
 
 	ClockOption* sco = new ClockOption (
@@ -130,8 +115,28 @@ SessionOptionEditor::SessionOptionEditor (Session* s)
 			    sigc::mem_fun (*_session_config, &SessionConfiguration::set_jack_time_master)
 			    ));
 
+	/* Sync */
+
+	add_option (_("Sync"), new OptionEditorHeading (_("A/V Synchronization")));
+	add_option (_("Sync"), new BoolOption (
+			    "use-video-file-fps",
+			    _("Use Video File's FPS Instead of Timecode Value for Timeline and Video Monitor."),
+			    sigc::mem_fun (*_session_config, &SessionConfiguration::get_use_video_file_fps),
+			    sigc::mem_fun (*_session_config, &SessionConfiguration::set_use_video_file_fps)
+			    ));
+
+	add_option (_("Sync"), new BoolOption (
+			    "videotimeline-pullup",
+			    _("Apply Pull-Up/Down to Video Timeline and Video Monitor (Unless using JACK-sync)."),
+			    sigc::mem_fun (*_session_config, &SessionConfiguration::get_videotimeline_pullup),
+			    sigc::mem_fun (*_session_config, &SessionConfiguration::set_videotimeline_pullup)
+			    ));
+
+	add_option (_("Sync"), new OptionEditorBlank ());
+
 	/* FADES */
 
+	add_option (_("Fades"), new OptionEditorHeading (_("Audio Fades")));
 	add_option (_("Fades"), new SpinOption<float> (
 		_("destructive-xfade-seconds"),
 		_("Destructive crossfade length"),
@@ -171,7 +176,7 @@ SessionOptionEditor::SessionOptionEditor (Session* s)
 
 	/* Media */
 
-	add_option (_("Media"), new OptionEditorHeading (_("Audio file format")));
+	add_option (_("Media"), new OptionEditorHeading (_("Audio File Format")));
 
 	ComboOption<SampleFormat>* sf = new ComboOption<SampleFormat> (
 		"native-file-data-format",
@@ -207,20 +212,20 @@ SessionOptionEditor::SessionOptionEditor (Session* s)
 
 	add_option (_("Media"), hf);
 
-	add_option (S_("Files|Locations"), new OptionEditorHeading (_("File locations")));
+	add_option (S_("Files|Locations"), new OptionEditorHeading (_("File Locations")));
 
-        SearchPathOption* spo = new SearchPathOption ("audio-search-path", _("Search for audio files in:"),
-						      _session->path(),
-                                                      sigc::mem_fun (*_session_config, &SessionConfiguration::get_audio_search_path),
-                                                      sigc::mem_fun (*_session_config, &SessionConfiguration::set_audio_search_path));
-        add_option (S_("Files|Locations"), spo);
+	SearchPathOption* spo = new SearchPathOption ("audio-search-path", _("Search for audio files in:"),
+			_session->path(),
+			sigc::mem_fun (*_session_config, &SessionConfiguration::get_audio_search_path),
+			sigc::mem_fun (*_session_config, &SessionConfiguration::set_audio_search_path));
+	add_option (S_("Files|Locations"), spo);
 
-        spo = new SearchPathOption ("midi-search-path", _("Search for MIDI files in:"),
-				    _session->path(),
-                                    sigc::mem_fun (*_session_config, &SessionConfiguration::get_midi_search_path),
-                                    sigc::mem_fun (*_session_config, &SessionConfiguration::set_midi_search_path));
+	spo = new SearchPathOption ("midi-search-path", _("Search for MIDI files in:"),
+			_session->path(),
+			sigc::mem_fun (*_session_config, &SessionConfiguration::get_midi_search_path),
+			sigc::mem_fun (*_session_config, &SessionConfiguration::set_midi_search_path));
 
-        add_option (S_("Files|Locations"), spo);
+	add_option (S_("Files|Locations"), spo);
 
 	/* File Naming  */
 
@@ -261,19 +266,23 @@ SessionOptionEditor::SessionOptionEditor (Session* s)
 
 	/* Monitoring */
 
-        add_option (_("Monitoring"), new BoolOption (
-			    "auto-input",
-			    _("Track Input Monitoring automatically follows transport state (\"auto-input\")"),
-			    sigc::mem_fun (*_session_config, &SessionConfiguration::get_auto_input),
-			    sigc::mem_fun (*_session_config, &SessionConfiguration::set_auto_input)
-			    ));
+	add_option (_("Monitoring"), new OptionEditorHeading (_("Monitoring")));
+	add_option (_("Monitoring"), new BoolOption (
+				"auto-input",
+				_("Track Input Monitoring automatically follows transport state (\"auto-input\")"),
+				sigc::mem_fun (*_session_config, &SessionConfiguration::get_auto_input),
+				sigc::mem_fun (*_session_config, &SessionConfiguration::set_auto_input)
+				));
+
+	add_option (_("Monitoring"), new BoolOption (
+				"have-monitor-section",
+				_("Use monitor section in this session"),
+				sigc::mem_fun (*this, &SessionOptionEditor::get_use_monitor_section),
+				sigc::mem_fun (*this, &SessionOptionEditor::set_use_monitor_section)
+				));
+
+	add_option (_("Monitoring"), new OptionEditorBlank ());
 
-        add_option (_("Monitoring"), new BoolOption (
-			    "have-monitor-section",
-			    _("Use monitor section in this session"),
-			    sigc::mem_fun (*this, &SessionOptionEditor::get_use_monitor_section),
-			    sigc::mem_fun (*this, &SessionOptionEditor::set_use_monitor_section)
-			    ));
 	/* Meterbridge */
 	add_option (_("Meterbridge"), new OptionEditorHeading (_("Route Display")));
 
@@ -337,6 +346,8 @@ SessionOptionEditor::SessionOptionEditor (Session* s)
 			    sigc::mem_fun (*_session_config, &SessionConfiguration::set_show_name_on_meterbridge)
 			    ));
 
+	add_option (_("Meterbridge"), new OptionEditorBlank ());
+
 	/* Misc */
 
 	add_option (_("Misc"), new OptionEditorHeading (_("MIDI Options")));
@@ -364,7 +375,7 @@ SessionOptionEditor::SessionOptionEditor (Session* s)
 
 	add_option (_("Misc"), li);
 
-	add_option (_("Misc"), new OptionEditorHeading (_("Glue to bars and beats")));
+	add_option (_("Misc"), new OptionEditorHeading (_("Glue to Bars and Beats")));
 
 	add_option (_("Misc"), new BoolOption (
 				"glue-new-markers-to-bars-and-beats",
@@ -380,12 +391,22 @@ SessionOptionEditor::SessionOptionEditor (Session* s)
 				sigc::mem_fun (*_session_config, &SessionConfiguration::set_glue_new_regions_to_bars_and_beats)
 				));
 
+	add_option (_("Misc"), new OptionEditorHeading (_("Metronome")));
+
+	add_option (_("Misc"), new BoolOption (
+				"count-in",
+				_("Always count-in when recording"),
+				sigc::mem_fun (*_session_config, &SessionConfiguration::get_count_in),
+				sigc::mem_fun (*_session_config, &SessionConfiguration::set_count_in)
+				));
+
 	add_option (_("Misc"), new OptionEditorHeading (_("Defaults")));
 
 	Gtk::Button* btn = Gtk::manage (new Gtk::Button (_("Use these settings as defaults")));
 	btn->signal_clicked().connect (sigc::mem_fun (*this, &SessionOptionEditor::save_defaults));
 	add_option (_("Misc"), new FooOption (btn));
 
+	set_current_page (_("Timecode"));
 }
 
 void
