@@ -28,7 +28,6 @@
 
 #include <boost/shared_ptr.hpp>
 #include <boost/weak_ptr.hpp>
-#include <boost/enable_shared_from_this.hpp>
 
 #include <glibmm/threads.h>
 #include "pbd/fastlog.h"
@@ -89,9 +88,7 @@ class LIBARDOUR_API Route : public Stripable,
                             public Soloable,
                             public Muteable,
                             public Monitorable,
-                            public Automatable,
-                            public RouteGroupMember,
-                            public boost::enable_shared_from_this<Route>
+                            public RouteGroupMember
 {
 public:
 
@@ -137,7 +134,7 @@ public:
 
 	virtual bool can_record() { return false; }
 
-	virtual void nonrealtime_handle_transport_stopped (bool abort, bool did_locate, bool flush_processors);
+	virtual void non_realtime_transport_stop (framepos_t now, bool flush);
 	virtual void realtime_handle_transport_stopped () {}
 	virtual void realtime_locate () {}
 	virtual void non_realtime_locate (framepos_t);
@@ -223,6 +220,8 @@ public:
 	RoutePinWindowProxy * pinmgr_proxy () const { return _pinmgr_proxy; }
 	void set_pingmgr_proxy (RoutePinWindowProxy* wp) { _pinmgr_proxy = wp ; }
 
+	boost::shared_ptr<AutomationControl> automation_control_recurse (PBD::ID const & id) const;
+
 	/* special processors */
 
 	boost::shared_ptr<InternalSend>     monitor_send() const { return _monitor_send; }
@@ -279,6 +278,7 @@ public:
 	void ab_plugins (bool forward);
 	void clear_processors (Placement);
 	void all_visible_processors_active (bool);
+	void move_instrument_down (bool postfader = false);
 
 	bool strict_io () const { return _strict_io; }
 	bool set_strict_io (bool);
@@ -359,6 +359,7 @@ public:
 
 	/** the processors have changed; the parameter indicates what changed */
 	PBD::Signal1<void,RouteProcessorChange> processors_changed;
+	PBD::Signal0<void> fan_out; // used to signal the GUI to fan-out (track-creation)
 	PBD::Signal1<void,void*> record_enable_changed;
 	PBD::Signal0<void> processor_latency_changed;
 	/** the metering point has changed */
@@ -503,12 +504,17 @@ public:
 	 */
 	uint32_t eq_band_cnt () const;
 	std::string eq_band_name (uint32_t) const;
+	boost::shared_ptr<AutomationControl> eq_enable_controllable () const;
 	boost::shared_ptr<AutomationControl> eq_gain_controllable (uint32_t band) const;
 	boost::shared_ptr<AutomationControl> eq_freq_controllable (uint32_t band) const;
 	boost::shared_ptr<AutomationControl> eq_q_controllable (uint32_t band) const;
 	boost::shared_ptr<AutomationControl> eq_shape_controllable (uint32_t band) const;
-	boost::shared_ptr<AutomationControl> eq_enable_controllable () const;
-	boost::shared_ptr<AutomationControl> eq_hpf_controllable () const;
+
+	//additional HP/LP filters
+	boost::shared_ptr<AutomationControl> filter_freq_controllable (bool hpf) const;
+	boost::shared_ptr<AutomationControl> filter_slope_controllable (bool) const;
+	boost::shared_ptr<AutomationControl> filter_enable_controllable (bool) const;
+
 
 	/* "well-known" controls for a compressor in this route. Any or all may
 	 * be null.
@@ -518,7 +524,7 @@ public:
 	boost::shared_ptr<AutomationControl> comp_speed_controllable () const;
 	boost::shared_ptr<AutomationControl> comp_mode_controllable () const;
 	boost::shared_ptr<AutomationControl> comp_makeup_controllable () const;
-	boost::shared_ptr<AutomationControl> comp_redux_controllable () const;
+	boost::shared_ptr<ReadOnlyControl>   comp_redux_controllable () const;
 
 	/* @param mode must be supplied by the comp_mode_controllable(). All other values
 	 * result in undefined behaviour
@@ -562,11 +568,11 @@ public:
 	void monitor_run (framepos_t start_frame, framepos_t end_frame,
 			pframes_t nframes, int declick);
 
-        bool slaved_to (boost::shared_ptr<VCA>) const;
-        bool slaved () const;
+	bool slaved_to (boost::shared_ptr<VCA>) const;
+	bool slaved () const;
 
-  protected:
-        friend class Session;
+protected:
+	friend class Session;
 
 	void catch_up_on_solo_mute_override ();
 	void set_listen (bool);
@@ -574,13 +580,11 @@ public:
 	void curve_reallocate ();
 	virtual void set_block_size (pframes_t nframes);
 
-  protected:
-        virtual framecnt_t check_initial_delay (framecnt_t nframes, framepos_t&) { return nframes; }
+	virtual framecnt_t check_initial_delay (framecnt_t nframes, framepos_t&) { return nframes; }
 
 	void fill_buffers_with_input (BufferSet& bufs, boost::shared_ptr<IO> io, pframes_t nframes);
 
-	void passthru (BufferSet&, framepos_t start_frame, framepos_t end_frame,
-			pframes_t nframes, int declick);
+	void passthru (BufferSet&, framepos_t start_frame, framepos_t end_frame, pframes_t nframes, int declick);
 
 	virtual void write_out_of_band_data (BufferSet& /* bufs */, framepos_t /* start_frame */, framepos_t /* end_frame */,
 					     framecnt_t /* nframes */) {}
@@ -655,7 +659,6 @@ public:
 
 	virtual ChanCount input_streams () const;
 
-protected:
 	virtual XMLNode& state(bool);
 
 	int configure_processors (ProcessorStreams*);
@@ -683,7 +686,9 @@ protected:
 
 	boost::shared_ptr<Processor> the_instrument_unlocked() const;
 
-  private:
+	SlavableControlList slavables () const;
+
+private:
 	int64_t _track_number;
 
 	int set_state_2X (const XMLNode&, int);
@@ -771,7 +776,7 @@ protected:
 
 	void reset_instrument_info ();
 
-        void solo_control_changed (bool self, PBD::Controllable::GroupControlDisposition);
+	void solo_control_changed (bool self, PBD::Controllable::GroupControlDisposition);
 };
 
 } // namespace ARDOUR
