@@ -114,6 +114,7 @@ class Bundle;
 class Butler;
 class Click;
 class ControllableDescriptor;
+class CoreSelection;
 class Diskstream;
 class ExportHandler;
 class ExportStatus;
@@ -199,7 +200,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 
 	virtual ~Session ();
 
-	static int get_info_from_path (const std::string& xmlpath, float& sample_rate, SampleFormat& data_format);
+	static int get_info_from_path (const std::string& xmlpath, float& sample_rate, SampleFormat& data_format, std::string& program_version);
 	static std::string get_snapshot_from_instant (const std::string& session_dir);
 
 	/** a monotonic counter used for naming user-visible things uniquely
@@ -248,7 +249,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	std::string format_audio_source_name (const std::string& legalized_base, uint32_t nchan, uint32_t chan, bool destructive, bool take_required, uint32_t cnt, bool related_exists);
 	std::string new_audio_source_path_for_embedded (const std::string& existing_path);
 	std::string new_audio_source_path (const std::string&, uint32_t nchans, uint32_t chan, bool destructive, bool take_required);
-	std::string new_midi_source_path (const std::string&);
+	std::string new_midi_source_path (const std::string&, bool need_source_lock = true);
 	/** create a new track or bus from a template (XML path)
 	 * @param how_many how many tracks or busses to create
 	 * @param template_path path to xml template file
@@ -258,7 +259,8 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	 */
 	RouteList new_route_from_template (uint32_t how_many, PresentationInfo::order_t insert_at, const std::string& template_path, const std::string& name, PlaylistDisposition pd = NewPlaylist);
 	RouteList new_route_from_template (uint32_t how_many, PresentationInfo::order_t insert_at, XMLNode&, const std::string& name, PlaylistDisposition pd = NewPlaylist);
-	std::vector<std::string> get_paths_for_new_sources (bool allow_replacing, const std::string& import_file_path, uint32_t channels);
+	std::vector<std::string> get_paths_for_new_sources (bool allow_replacing, const std::string& import_file_path, 
+	                                                    uint32_t channels, std::vector<std::string> const & smf_track_names);
 
 	int bring_all_sources_into_session (boost::function<void(uint32_t,uint32_t,std::string)> callback);
 
@@ -288,6 +290,8 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 		return routes.reader ();
 	}
 
+	CoreSelection& selection () { return *_selection; }
+
 	/* because the set of Stripables consists of objects managed
 	 * independently, in multiple containers within the Session (or objects
 	 * owned by the session), we fill out a list in-place rather than
@@ -296,10 +300,12 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	 */
 
 	void get_stripables (StripableList&) const;
+	StripableList get_stripables () const;
 	boost::shared_ptr<RouteList> get_tracks() const;
 	boost::shared_ptr<RouteList> get_routes_with_internal_returns() const;
 	boost::shared_ptr<RouteList> get_routes_with_regions_at (framepos_t const) const;
 
+	uint32_t nstripables (bool with_monitor = false) const;
 	uint32_t nroutes() const { return routes.reader()->size(); }
 	uint32_t ntracks () const;
 	uint32_t nbusses () const;
@@ -308,10 +314,6 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 		return _bundles.reader ();
 	}
 
-	struct LIBARDOUR_API RoutePublicOrderSorter {
-		bool operator() (boost::shared_ptr<Route>, boost::shared_ptr<Route> b);
-	};
-
 	void notify_presentation_info_change ();
 
 	template<class T> void foreach_route (T *obj, void (T::*func)(Route&), bool sort = true);
@@ -322,6 +324,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	bool io_name_is_legal (const std::string&) const;
 	boost::shared_ptr<Route> route_by_name (std::string) const;
 	boost::shared_ptr<Route> route_by_id (PBD::ID) const;
+	boost::shared_ptr<Stripable> stripable_by_id (PBD::ID) const;
 	boost::shared_ptr<Stripable> get_remote_nth_stripable (PresentationInfo::order_t n, PresentationInfo::Flag) const;
 	boost::shared_ptr<Route> get_remote_nth_route (PresentationInfo::order_t n) const;
 	boost::shared_ptr<Route> route_by_selected_count (uint32_t cnt) const;
@@ -348,7 +351,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	}
 
 	bool record_enabling_legal () const;
-	void maybe_enable_record ();
+	void maybe_enable_record (bool rt_context = false);
 	void disable_record (bool rt_context, bool force = false);
 	void step_back_from_record ();
 
@@ -450,7 +453,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	framecnt_t nominal_frame_rate () const { return _nominal_frame_rate; }
 	framecnt_t frames_per_hour () const { return _frames_per_hour; }
 
-	double frames_per_timecode_frame() const { return _frames_per_timecode_frame; }
+	double samples_per_timecode_frame() const { return _samples_per_timecode_frame; }
 	framecnt_t timecode_frames_per_hour() const { return _timecode_frames_per_hour; }
 
 	MIDI::byte get_mtc_timecode_bits() const {
@@ -471,6 +474,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	void set_auto_punch_location (Location *);
 	void set_auto_loop_location (Location *);
 	void set_session_extents (framepos_t start, framepos_t end);
+	bool end_is_free () const { return _session_range_end_is_free; }
 	void set_end_is_free (bool);
 	int location_name(std::string& result, std::string base = std::string(""));
 
@@ -562,6 +566,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	XMLNode& get_state();
 	int      set_state(const XMLNode& node, int version); // not idempotent
 	XMLNode& get_template();
+	bool     export_track_state (boost::shared_ptr<RouteList> rl, const std::string& path);
 
 	/// The instant xml file is written to the session directory
 	void add_instant_xml (XMLNode&, bool write_to_config = true);
@@ -617,7 +622,9 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 			bool _reconfigure_on_delete;
 	};
 
+	RouteGroup* new_route_group (const std::string&);
 	void add_route_group (RouteGroup *);
+	void remove_route_group (RouteGroup* rg) { if (rg) remove_route_group (*rg); }
 	void remove_route_group (RouteGroup&);
 	void reorder_route_groups (std::list<RouteGroup*>);
 
@@ -651,7 +658,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 		);
 
 	std::list<boost::shared_ptr<MidiTrack> > new_midi_track (
-		const ChanCount& input, const ChanCount& output,
+		const ChanCount& input, const ChanCount& output, bool strict_io,
 		boost::shared_ptr<PluginInfo> instrument,
 		Plugin::PresetRecord* pset,
 		RouteGroup* route_group, uint32_t how_many, std::string name_template,
@@ -660,7 +667,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 		);
 
 	RouteList new_audio_route (int input_channels, int output_channels, RouteGroup* route_group, uint32_t how_many, std::string name_template, PresentationInfo::Flag, PresentationInfo::order_t);
-	RouteList new_midi_route (RouteGroup* route_group, uint32_t how_many, std::string name_template, boost::shared_ptr<PluginInfo> instrument, Plugin::PresetRecord*, PresentationInfo::Flag, PresentationInfo::order_t);
+	RouteList new_midi_route (RouteGroup* route_group, uint32_t how_many, std::string name_template, bool strict_io, boost::shared_ptr<PluginInfo> instrument, Plugin::PresetRecord*, PresentationInfo::Flag, PresentationInfo::order_t);
 
 	void remove_routes (boost::shared_ptr<RouteList>);
 	void remove_route (boost::shared_ptr<Route>);
@@ -677,7 +684,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 
 	framepos_t transport_frame () const {return _transport_frame; }
 	framepos_t record_location () const {return _last_record_location; }
-	framepos_t audible_frame () const;
+	framepos_t audible_frame (bool* latent_locate = NULL) const;
 	framepos_t requested_return_frame() const { return _requested_return_frame; }
 	void set_requested_return_frame(framepos_t return_to);
 
@@ -716,9 +723,9 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	bool   synced_to_mtc () const { return config.get_external_sync() && Config->get_sync_source() == MTC && g_atomic_int_get (const_cast<gint*>(&_mtc_active)); }
 	bool   synced_to_ltc () const { return config.get_external_sync() && Config->get_sync_source() == LTC && g_atomic_int_get (const_cast<gint*>(&_ltc_active)); }
 
-	double transport_speed() const { return _transport_speed; }
+	double transport_speed() const { return _count_in_samples > 0 ? 0. : _transport_speed; }
 	bool   transport_stopped() const { return _transport_speed == 0.0; }
-	bool   transport_rolling() const { return _transport_speed != 0.0; }
+	bool   transport_rolling() const { return _transport_speed != 0.0 && _count_in_samples == 0; }
 
 	bool silent () { return _silent; }
 
@@ -794,7 +801,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 
 	boost::shared_ptr<Source> source_by_id (const PBD::ID&);
 	boost::shared_ptr<AudioFileSource> audio_source_by_path_and_channel (const std::string&, uint16_t) const;
-	boost::shared_ptr<MidiSource> midi_source_by_path (const std::string&) const;
+	boost::shared_ptr<MidiSource> midi_source_by_path (const std::string&, bool need_source_lock) const;
 	uint32_t count_sources_by_origin (const std::string&);
 
 	void add_playlist (boost::shared_ptr<Playlist>, bool unused = false);
@@ -819,6 +826,8 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	uint32_t registered_lua_function_count () const { return _n_lua_scripts; }
 	void scripts_changed (); // called from lua, updates _n_lua_scripts
 
+	PBD::Signal0<void> LuaScriptsChanged;
+
 	/* flattening stuff */
 
 	boost::shared_ptr<Region> write_one_track (Track&, framepos_t start, framepos_t end,
@@ -829,6 +838,9 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 
 	/* session-wide solo/mute/rec-enable */
 
+	bool muted() const;
+	std::vector<boost::weak_ptr<AutomationControl> > cancel_all_mute ();
+
 	bool soloing() const { return _non_soloed_outs_muted; }
 	bool listening() const { return _listen_cnt > 0; }
 	bool solo_isolated() const { return _solo_isolated_cnt > 0; }
@@ -847,12 +859,14 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 
 	PBD::Signal1<void,bool> SoloActive;
 	PBD::Signal0<void> SoloChanged;
+	PBD::Signal0<void> MuteChanged;
 	PBD::Signal0<void> IsolatedChanged;
 	PBD::Signal0<void> MonitorChanged;
 
 	PBD::Signal0<void> session_routes_reconnected;
 
 	/* monitor/master out */
+	int add_master_bus (ChanCount const&);
 
 	void add_monitor_section ();
 	void reset_monitor_section ();
@@ -862,6 +876,9 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	boost::shared_ptr<Route> monitor_out() const { return _monitor_out; }
 	boost::shared_ptr<Route> master_out() const { return _master_out; }
 
+	PresentationInfo::order_t master_order_key () const { return _master_out ? _master_out->presentation_info ().order () : -1; }
+	bool ensure_stripable_sort_order ();
+
 	void globally_add_internal_sends (boost::shared_ptr<Route> dest, Placement p, bool);
 	void globally_set_send_gains_from_track (boost::shared_ptr<Route> dest);
 	void globally_set_send_gains_to_zero (boost::shared_ptr<Route> dest);
@@ -994,6 +1011,18 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 
 	void maybe_update_session_range (framepos_t, framepos_t);
 
+	/* preroll */
+	framecnt_t preroll_samples (framepos_t) const;
+
+	void request_preroll_record_punch (framepos_t start, framecnt_t preroll);
+	void request_preroll_record_trim (framepos_t start, framecnt_t preroll);
+	void request_count_in_record ();
+
+	framepos_t preroll_record_punch_pos () const { return _preroll_record_punch_pos; }
+	bool preroll_record_punch_enabled () const { return _preroll_record_punch_pos >= 0; }
+
+	framecnt_t preroll_record_trim_len () const { return _preroll_record_trim_len; }
+
 	/* temporary hacks to allow selection to be pushed from GUI into backend.
 	   Whenever we move the selection object into libardour, these will go away.
 	 */
@@ -1007,6 +1036,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	gain_t* gain_automation_buffer () const;
 	gain_t* trim_automation_buffer () const;
 	gain_t* send_gain_automation_buffer () const;
+	gain_t* scratch_automation_buffer () const;
 	pan_t** pan_automation_buffer () const;
 
 	void ensure_buffer_set (BufferSet& buffers, const ChanCount& howmany);
@@ -1037,6 +1067,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	boost::shared_ptr<Processor> processor_by_id (PBD::ID) const;
 
 	boost::shared_ptr<PBD::Controllable> controllable_by_id (const PBD::ID&);
+	boost::shared_ptr<AutomationControl> automation_control_by_id (const PBD::ID&);
 	boost::shared_ptr<PBD::Controllable> controllable_by_descriptor (const ARDOUR::ControllableDescriptor&);
 
 	void add_controllable (boost::shared_ptr<PBD::Controllable>);
@@ -1127,6 +1158,10 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	*/
 	static PBD::Signal3<int,Session*,std::string,DataType> MissingFile;
 
+	void set_missing_file_replacement (const std::string& mfr) {
+		_missing_file_replacement = mfr;
+	}
+
 	/** Emitted when the session wants Ardour to quit */
 	static PBD::Signal0<void> Quit;
 
@@ -1242,6 +1277,8 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	bool                    _under_nsm_control;
 	unsigned int            _xrun_count;
 
+	std::string             _missing_file_replacement;
+
 	void mtc_status_changed (bool);
 	PBD::ScopedConnection mtc_status_connection;
 	void ltc_status_changed (bool);
@@ -1360,7 +1397,6 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	 *  know when to send full MTC messages every so often.
 	 */
 	pframes_t               _pframes_since_last_mtc;
-	bool                     session_midi_feedback;
 	bool                     play_loop;
 	bool                     loop_changing;
 	framepos_t               last_loopend;
@@ -1419,7 +1455,6 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	static const PostTransportWork ProcessCannotProceedMask =
 		PostTransportWork (
 			PostTransportInputChange|
-			PostTransportSpeed|
 			PostTransportReverse|
 			PostTransportCurveRealloc|
 			PostTransportAudition|
@@ -1520,7 +1555,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	void auto_connect_thread_terminate ();
 
 	pthread_t       _auto_connect_thread;
-	bool            _ac_thread_active;
+	gint            _ac_thread_active;
 	pthread_mutex_t _auto_connect_mutex;
 	pthread_cond_t  _auto_connect_cond;
 
@@ -1602,7 +1637,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	Timecode::Time transmitting_timecode_time;
 	int next_quarter_frame_to_send;
 
-	double _frames_per_timecode_frame; /* has to be floating point because of drop frame */
+	double _samples_per_timecode_frame; /* has to be floating point because of drop frame */
 	framecnt_t _frames_per_hour;
 	framecnt_t _timecode_frames_per_hour;
 
@@ -1643,6 +1678,8 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	void ltc_tx_recalculate_position();
 	void ltc_tx_parse_offset();
 	void ltc_tx_send_time_code_for_cycle (framepos_t, framepos_t, double, double, pframes_t nframes);
+	PBD::ScopedConnectionList ltc_tx_connections;
+
 
 	void reset_record_status ();
 
@@ -1691,7 +1728,6 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 
 	TempoMap    *_tempo_map;
 	void          tempo_map_changed (const PBD::PropertyChange&);
-	void          gui_tempo_map_changed ();
 
 	/* edit/mix groups */
 
@@ -1868,12 +1904,19 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 
 	void  update_latency (bool playback);
 
-	XMLNode& state(bool);
+	enum snapshot_t {
+		NormalSave,
+		SnapshotKeep,
+		SwitchToSnapshot
+	};
+
+	XMLNode& state(bool, snapshot_t snapshot_type = NormalSave);
 
 	/* click track */
 	typedef std::list<Click*> Clicks;
 	Clicks                  clicks;
 	bool                   _clicking;
+	bool                   _click_rec_only;
 	boost::shared_ptr<IO>  _click_io;
 	boost::shared_ptr<Amp> _click_gain;
 	Sample*                 click_data;
@@ -1893,6 +1936,9 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	void   setup_click_sounds (Sample**, Sample const *, framecnt_t*, framecnt_t, std::string const &);
 	void   clear_clicks ();
 	void   click (framepos_t start, framecnt_t nframes);
+	void   run_click (framepos_t start, framecnt_t nframes);
+	void   add_click (framepos_t pos, bool emphasis);
+	framecnt_t _count_in_samples;
 
 	std::vector<Route*> master_outs;
 
@@ -1909,6 +1955,13 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	Evoral::Range<framepos_t> _range_selection;
 	Evoral::Range<framepos_t> _object_selection;
 
+	void unset_preroll_record_punch ();
+	void unset_preroll_record_trim ();
+
+	framepos_t _preroll_record_punch_pos;
+	framecnt_t _preroll_record_trim_len;
+	bool _count_in_once;
+
 	/* main outs */
 	uint32_t main_outs;
 
@@ -1917,17 +1970,6 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 
 	void auto_connect_master_bus ();
 
-	/* Windows VST support */
-
-	long _windows_vst_callback (
-		WindowsVSTPlugin*,
-		long opcode,
-		long index,
-		long value,
-		void* ptr,
-		float opt
-		);
-
 	int find_all_sources (std::string path, std::set<std::string>& result);
 	int find_all_sources_across_snapshots (std::set<std::string>& result, bool exclude_this_snapshot);
 
@@ -2048,7 +2090,6 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 
 	void save_as_bring_callback (uint32_t, uint32_t, std::string);
 
-	static int get_session_info_from_path (XMLTree& state_tree, const std::string& xmlpath);
 	static const uint32_t session_end_shift;
 
 	std::string _template_state_dir;
@@ -2063,6 +2104,8 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	void rewire_selected_midi (boost::shared_ptr<MidiTrack>);
 	void rewire_midi_selection_ports ();
 	boost::weak_ptr<MidiTrack> current_midi_target;
+
+	CoreSelection* _selection;
 };
 
 
