@@ -22,7 +22,6 @@
 #include <glibmm/fileutils.h>
 
 #include "pbd/gstdio_compat.h"
-#include "pbd/locale_guard.h"
 #include "pbd/pthread_utils.h"
 
 #include "ardour/audio_buffer.h"
@@ -87,6 +86,7 @@ LuaProc::LuaProc (const LuaProc &other)
 #endif
 	, _lua_dsp (0)
 	, _script (other.script ())
+	, _origin (other._origin)
 	, _lua_does_channelmapping (false)
 	, _lua_has_inline_display (false)
 	, _designated_bypass_port (UINT32_MAX)
@@ -163,7 +163,7 @@ LuaProc::init ()
 	lua_setglobal (L, "self");
 
 	// sandbox
-	lua.do_command ("io = nil os = nil loadfile = nil require = nil dofile = nil package = nil debug = nil");
+	lua.sandbox (true);
 #if 0
 	lua.do_command ("for n in pairs(_G) do print(n) end print ('----')"); // print global env
 #endif
@@ -173,6 +173,9 @@ LuaProc::init ()
 boost::weak_ptr<Route>
 LuaProc::route () const
 {
+	if (!_owner) {
+		return boost::weak_ptr<Route>();
+	}
 	return static_cast<Route*>(_owner)->weakroute ();
 }
 
@@ -667,7 +670,7 @@ LuaProc::connect_and_run (BufferSet& bufs,
 				if (valid) {
 					for (MidiBuffer::iterator m = bufs.get_midi(idx).begin();
 							m != bufs.get_midi(idx).end(); ++m, ++e) {
-						const Evoral::MIDIEvent<framepos_t> ev(*m, false);
+						const Evoral::Event<framepos_t> ev(*m, false);
 						luabridge::LuaRef lua_midi_data (luabridge::newTable (L));
 						const uint8_t* data = ev.buffer();
 						for (uint32_t i = 0; i < ev.size(); ++i) {
@@ -755,24 +758,21 @@ void
 LuaProc::add_state (XMLNode* root) const
 {
 	XMLNode*    child;
-	char        buf[32];
-	LocaleGuard lg;
 
 	gchar* b64 = g_base64_encode ((const guchar*)_script.c_str (), _script.size ());
 	std::string b64s (b64);
 	g_free (b64);
 	XMLNode* script_node = new XMLNode (X_("script"));
-	script_node->add_property (X_("lua"), LUA_VERSION);
+	script_node->set_property (X_("lua"), LUA_VERSION);
+	script_node->set_property (X_("origin"), _origin);
 	script_node->add_content (b64s);
 	root->add_child_nocopy (*script_node);
 
 	for (uint32_t i = 0; i < parameter_count(); ++i) {
 		if (parameter_is_input(i) && parameter_is_control(i)) {
 			child = new XMLNode("Port");
-			snprintf(buf, sizeof(buf), "%u", i);
-			child->add_property("id", std::string(buf));
-			snprintf(buf, sizeof(buf), "%+f", _shadow_data[i]);
-			child->add_property("value", std::string(buf));
+			child->set_property("id", i);
+			child->set_property("value", _shadow_data[i]);
 			root->add_child_nocopy(*child);
 		}
 	}
@@ -787,6 +787,10 @@ LuaProc::set_script_from_state (const XMLNode& node)
 	}
 
 	if ((child = node.child (X_("script"))) != 0) {
+		XMLProperty const* prop;
+		if ((prop = node.property ("origin")) != 0) {
+			_origin = prop->value();
+		}
 		for (XMLNodeList::const_iterator n = child->children ().begin (); n != child->children ().end (); ++n) {
 			if (!(*n)->is_content ()) { continue; }
 			gsize size;
@@ -819,14 +823,9 @@ LuaProc::set_state (const XMLNode& node, int version)
 {
 #ifndef NO_PLUGIN_STATE
 	XMLNodeList nodes;
-	XMLProperty const * prop;
 	XMLNodeConstIterator iter;
 	XMLNode *child;
-	const char *value;
-	const char *port;
-	uint32_t port_id;
 #endif
-	LocaleGuard lg;
 
 	if (_script.empty ()) {
 		if (set_script_from_state (node)) {
@@ -843,20 +842,21 @@ LuaProc::set_state (const XMLNode& node, int version)
 	nodes = node.children ("Port");
 	for (iter = nodes.begin(); iter != nodes.end(); ++iter) {
 		child = *iter;
-		if ((prop = child->property("id")) != 0) {
-			port = prop->value().c_str();
-		} else {
+
+		uint32_t port_id;
+		float value;
+
+		if (!child->get_property("id", port_id)) {
 			warning << _("LuaProc: port has no symbol, ignored") << endmsg;
 			continue;
 		}
-		if ((prop = child->property("value")) != 0) {
-			value = prop->value().c_str();
-		} else {
+
+		if (!child->get_property("value", value)) {
 			warning << _("LuaProc: port has no value, ignored") << endmsg;
 			continue;
 		}
-		sscanf (port, "%" PRIu32, &port_id);
-		set_parameter (port_id, atof(value));
+
+		set_parameter (port_id, value);
 	}
 #endif
 
@@ -1112,20 +1112,24 @@ LuaProc::load_preset (PresetRecord r)
 
 	XMLNode* root = t->root ();
 	for (XMLNodeList::const_iterator i = root->children().begin(); i != root->children().end(); ++i) {
-		XMLProperty const * label = (*i)->property (X_("label"));
-		assert (label);
-		if (label->value() != r.label) {
+		std::string str;
+		if (!(*i)->get_property (X_("label"), str)) {
+			assert (false);
+		}
+		if (str != r.label) {
 			continue;
 		}
 
 		for (XMLNodeList::const_iterator j = (*i)->children().begin(); j != (*i)->children().end(); ++j) {
 			if ((*j)->name() == X_("Parameter")) {
-				XMLProperty const * index = (*j)->property (X_("index"));
-				XMLProperty const * value = (*j)->property (X_("value"));
-				assert (index);
-				assert (value);
-				LocaleGuard lg;
-				set_parameter (atoi (index->value().c_str()), atof (value->value().c_str ()));
+				uint32_t index;
+				float value;
+				if (!(*j)->get_property (X_("index"), index) ||
+				    !(*j)->get_property (X_("value"), value)) {
+					assert (false);
+				}
+				set_parameter (index, value);
+				PresetPortSetValue (index, value); /* EMIT SIGNAL */
 			}
 		}
 		return Plugin::load_preset(r);
@@ -1141,17 +1145,20 @@ LuaProc::do_save_preset (std::string name) {
 		return "";
 	}
 
+	// prevent dups -- just in case
+	t->root()->remove_nodes_and_delete (X_("label"), name);
+
 	std::string uri (preset_name_to_uri (name));
 
 	XMLNode* p = new XMLNode (X_("Preset"));
-	p->add_property (X_("uri"), uri);
-	p->add_property (X_("label"), name);
+	p->set_property (X_("uri"), uri);
+	p->set_property (X_("label"), name);
 
 	for (uint32_t i = 0; i < parameter_count(); ++i) {
 		if (parameter_is_input (i)) {
 			XMLNode* c = new XMLNode (X_("Parameter"));
-			c->add_property (X_("index"), string_compose ("%1", i));
-			c->add_property (X_("value"), string_compose ("%1", get_parameter (i)));
+			c->set_property (X_("index"), i);
+			c->set_property (X_("value"), get_parameter (i));
 			p->add_child_nocopy (*c);
 		}
 	}
@@ -1184,14 +1191,14 @@ LuaProc::find_presets ()
 	if (t) {
 		XMLNode* root = t->root ();
 		for (XMLNodeList::const_iterator i = root->children().begin(); i != root->children().end(); ++i) {
+			std::string uri;
+			std::string label;
 
-			XMLProperty const * uri = (*i)->property (X_("uri"));
-			XMLProperty const * label = (*i)->property (X_("label"));
+			if (!(*i)->get_property (X_("uri"), uri) || !(*i)->get_property (X_("label"), label)) {
+				assert (false);
+			}
 
-			assert (uri);
-			assert (label);
-
-			PresetRecord r (uri->value(), label->value(), true);
+			PresetRecord r (uri, label, true);
 			_presets.insert (make_pair (r.uri, r));
 		}
 	}
@@ -1236,7 +1243,9 @@ LuaPluginInfo::load (Session& session)
 	}
 
 	try {
-		PluginPtr plugin (new LuaProc (session.engine (), session, script));
+		LuaProc* lp = new LuaProc (session.engine (), session, script);
+		lp->set_origin (path);
+		PluginPtr plugin (lp);
 		return plugin;
 	} catch (failed_constructor& err) {
 		;
