@@ -40,6 +40,7 @@
 #include "ardour/audioengine.h"
 #include "ardour/butler.h"
 #include "ardour/debug.h"
+#include "ardour/evoral_types_convert.h"
 #include "ardour/io.h"
 #include "ardour/midi_diskstream.h"
 #include "ardour/midi_model.h"
@@ -288,6 +289,7 @@ MidiDiskstream::use_copy_playlist ()
 	}
 }
 
+#ifdef XXX_OLD_DESTRUCTIVE_API_XXX
 /** Overloaded from parent to die horribly
  */
 int
@@ -295,6 +297,7 @@ MidiDiskstream::set_destructive (bool yn)
 {
 	return yn ? -1 : 0;
 }
+#endif
 
 void
 MidiDiskstream::set_note_mode (NoteMode m)
@@ -365,7 +368,7 @@ MidiDiskstream::process (BufferSet& bufs, framepos_t transport_frame, pframes_t
 
 	adjust_capture_position = 0;
 
-	if (nominally_recording || (re && was_recording && _session.get_record_enabled() && _session.config.get_punch_in())) {
+	if (nominally_recording || (re && was_recording && _session.get_record_enabled() && (_session.config.get_punch_in() || _session.preroll_record_punch_enabled()))) {
 		Evoral::OverlapType ot = Evoral::coverage (first_recordable_frame, last_recordable_frame, transport_frame, transport_frame + nframes);
 		// XXX should this be transport_frame + nframes - 1 ? coverage() expects its parameter ranges to include their end points
 
@@ -409,7 +412,7 @@ MidiDiskstream::process (BufferSet& bufs, framepos_t transport_frame, pframes_t
 		MidiChannelFilter* filter = mt ? &mt->capture_filter() : NULL;
 
 		for (MidiBuffer::iterator i = buf.begin(); i != buf.end(); ++i) {
-			Evoral::MIDIEvent<MidiBuffer::TimeType> ev(*i, false);
+			Evoral::Event<MidiBuffer::TimeType> ev(*i, false);
 			if (ev.time() + rec_offset > rec_nframes) {
 				break;
 			}
@@ -445,7 +448,7 @@ MidiDiskstream::process (BufferSet& bufs, framepos_t transport_frame, pframes_t
 			}
 
 			if (!filter || !filter->filter(ev.buffer(), ev.size())) {
-				_capture_buf->write(event_time, ev.type(), ev.size(), ev.buffer());
+				_capture_buf->write(event_time, ev.event_type(), ev.size(), ev.buffer());
 			}
 		}
 		g_atomic_int_add(const_cast<gint*>(&_frames_pending_write), nframes);
@@ -1033,6 +1036,7 @@ MidiDiskstream::transport_stopped_wallclock (struct tm& /*when*/, time_t /*twhen
 				initial_capture = capture_info.front()->start;
 			}
 
+			const framepos_t preroll_off = _session.preroll_record_trim_len ();
 			for (ci = capture_info.begin(); ci != capture_info.end(); ++ci) {
 
 				string region_name;
@@ -1056,6 +1060,9 @@ MidiDiskstream::transport_stopped_wallclock (struct tm& /*when*/, time_t /*twhen
 
 					boost::shared_ptr<Region> rx (RegionFactory::create (srcs, plist));
 					region = boost::dynamic_pointer_cast<MidiRegion> (rx);
+					if (preroll_off > 0) {
+						region->trim_front ((*ci)->start - initial_capture + preroll_off);
+					}
 				}
 
 				catch (failed_constructor& err) {
@@ -1066,7 +1073,7 @@ MidiDiskstream::transport_stopped_wallclock (struct tm& /*when*/, time_t /*twhen
 				// cerr << "add new region, buffer position = " << buffer_position << " @ " << (*ci)->start << endl;
 
 				i_am_the_modifier++;
-				_playlist->add_region (region, (*ci)->start);
+				_playlist->add_region (region, (*ci)->start + preroll_off, 1, non_layered());
 				i_am_the_modifier--;
 			}
 
@@ -1220,8 +1227,6 @@ XMLNode&
 MidiDiskstream::get_state ()
 {
 	XMLNode& node (Diskstream::get_state());
-	char buf[64];
-	LocaleGuard lg;
 
 	if (_write_source && _session.get_record_enabled()) {
 
@@ -1229,20 +1234,21 @@ MidiDiskstream::get_state ()
 		XMLNode* cs_grandchild;
 
 		cs_grandchild = new XMLNode (X_("file"));
-		cs_grandchild->add_property (X_("path"), _write_source->path());
+		cs_grandchild->set_property (X_("path"), _write_source->path());
 		cs_child->add_child_nocopy (*cs_grandchild);
 
 		/* store the location where capture will start */
 
 		Location* pi;
 
-		if (_session.config.get_punch_in() && ((pi = _session.locations()->auto_punch_location()) != 0)) {
-			snprintf (buf, sizeof (buf), "%" PRId64, pi->start());
+		if (_session.preroll_record_punch_enabled ()) {
+			cs_child->set_property (X_("at"), _session.preroll_record_punch_pos());
+		} else if (_session.config.get_punch_in() && ((pi = _session.locations()->auto_punch_location()) != 0)) {
+			cs_child->set_property (X_("at"), pi->start());
 		} else {
-			snprintf (buf, sizeof (buf), "%" PRId64, _session.transport_frame());
+			cs_child->set_property (X_("at"), _session.transport_frame());
 		}
 
-		cs_child->add_property (X_("at"), buf);
 		node.add_child_nocopy (*cs_child);
 	}
 
@@ -1255,7 +1261,6 @@ MidiDiskstream::set_state (const XMLNode& node, int version)
 	XMLNodeList nlist = node.children();
 	XMLNodeIterator niter;
 	XMLNode* capture_pending_node = 0;
-	LocaleGuard lg;
 
 	/* prevent write sources from being created */
 
