@@ -25,7 +25,7 @@
 
 #include "pbd/failed_constructor.h"
 #include "pbd/xml++.h"
-#include "pbd/convert.h"
+#include "pbd/types_convert.h"
 
 #include "ardour/audio_buffer.h"
 #include "ardour/automation_list.h"
@@ -98,6 +98,9 @@ PluginInsert::PluginInsert (Session& s, boost::shared_ptr<Plugin> plug)
 
 PluginInsert::~PluginInsert ()
 {
+	for (CtrlOutMap::const_iterator i = _control_outputs.begin(); i != _control_outputs.end(); ++i) {
+		boost::dynamic_pointer_cast<ReadOnlyControl>(i->second)->drop_references ();
+	}
 }
 
 void
@@ -430,6 +433,13 @@ PluginInsert::has_output_presets (ChanCount in, ChanCount out)
 			return false;
 		}
 	}
+
+	if (ppc.size () == 1 && ppc.find (0) != ppc.end () && !_plugins[0]->get_info ()->reconfigurable_io ()) {
+		// some midi-sequencer (e.g. QMidiArp) or other midi-out plugin
+		// pretending to be an "Instrument"
+		return false;
+	}
+
 	if (!is_instrument ()) {
 			return false;
 	}
@@ -445,14 +455,19 @@ PluginInsert::create_automatable_parameters ()
 	set<Evoral::Parameter> a = _plugins.front()->automatable ();
 
 	for (uint32_t i = 0; i < plugin->parameter_count(); ++i) {
-		if (!plugin->parameter_is_control (i) || !plugin->parameter_is_input (i)) {
+		if (!plugin->parameter_is_control (i)) {
 			continue;
 		}
-		Evoral::Parameter param (PluginAutomation, 0, i);
 
 		ParameterDescriptor desc;
 		plugin->get_parameter_descriptor(i, desc);
 
+		if (!plugin->parameter_is_input (i)) {
+			_control_outputs[i] = boost::shared_ptr<ReadOnlyControl> (new ReadOnlyControl (plugin, desc, i));
+			continue;
+		}
+		Evoral::Parameter param (PluginAutomation, 0, i);
+
 		const bool automatable = a.find(param) != a.end();
 
 		if (automatable) {
@@ -483,6 +498,21 @@ PluginInsert::create_automatable_parameters ()
 
 	_bypass_port = plugin->designated_bypass_port ();
 
+	/* special case VST effSetBypass */
+	if (_bypass_port == UINT32_MAX -1) {
+		// emulate VST Bypass
+		Evoral::Parameter param (PluginAutomation, 0, _bypass_port);
+		ParameterDescriptor desc;
+		desc.label = _("Plugin Enable");
+		desc.toggled  = true;
+		desc.normal = 1;
+		desc.lower  = 0;
+		desc.upper  = 1;
+		boost::shared_ptr<AutomationList> list(new AutomationList(param, desc));
+		boost::shared_ptr<AutomationControl> c (new PluginControl(this, param, desc, list));
+		add_control (c);
+	}
+
 	if (_bypass_port != UINT32_MAX) {
 		boost::shared_ptr<AutomationControl> ac = automation_control (Evoral::Parameter (PluginAutomation, 0, _bypass_port));
 		if (0 == (ac->flags () & Controllable::NotAutomatable)) {
@@ -490,7 +520,9 @@ PluginInsert::create_automatable_parameters ()
 			ac->Changed.connect_same_thread (*this, boost::bind (&PluginInsert::enable_changed, this));
 		}
 	}
+	plugin->PresetPortSetValue.connect_same_thread (*this, boost::bind (&PluginInsert::preset_load_set_value, this, _1, _2));
 }
+
 /** Called when something outside of this host has modified a plugin
  * parameter. Responsible for propagating the change to two places:
  *
@@ -609,7 +641,27 @@ PluginInsert::enable (bool yn)
 			activate ();
 		}
 		boost::shared_ptr<AutomationControl> ac = automation_control (Evoral::Parameter (PluginAutomation, 0, _bypass_port));
-		ac->set_value (yn ? 1.0 : 0.0, Controllable::NoGroup);
+		const double val = yn ? 1.0 : 0.0;
+		ac->set_value (val, Controllable::NoGroup);
+
+#ifdef ALLOW_VST_BYPASS_TO_FAIL // yet unused, see also vst_plugin.cc
+		/* special case VST.. bypass may fail */
+		if (_bypass_port == UINT32_MAX - 1) {
+			/* check if bypass worked */
+			if (ac->get_value () != val) {
+				warning << _("PluginInsert: VST Bypass failed, falling back to host bypass.") << endmsg;
+				// set plugin to enabled (not-byassed)
+				ac->set_value (1.0, Controllable::NoGroup);
+				// ..and use host-provided hard-bypass
+				if (yn) {
+					activate ();
+				} else {
+					deactivate ();
+				}
+				return;
+			}
+		}
+#endif
 		ActiveChanged ();
 	}
 }
@@ -649,6 +701,23 @@ PluginInsert::bypassable_changed ()
 	BypassableChanged ();
 }
 
+void
+PluginInsert::preset_load_set_value (uint32_t p, float v)
+{
+	boost::shared_ptr<AutomationControl> ac = automation_control (Evoral::Parameter(PluginAutomation, 0, p));
+	if (!ac) {
+		return;
+	}
+
+	if (ac->automation_state() & Play) {
+		return;
+	}
+
+	start_touch (p);
+	ac->set_value (v, Controllable::NoGroup);
+	end_touch (p);
+}
+
 void
 PluginInsert::inplace_silence_unconnected (BufferSet& bufs, const PinMappings& out_map, framecnt_t nframes, framecnt_t offset) const
 {
@@ -1059,7 +1128,10 @@ PluginInsert::bypass (BufferSet& bufs, pframes_t nframes)
 void
 PluginInsert::silence (framecnt_t nframes, framepos_t start_frame)
 {
+	automation_run (start_frame, nframes); // evaluate automation only
+
 	if (!active ()) {
+		// XXX delaybuffers need to be offset by nframes
 		return;
 	}
 
@@ -1093,14 +1165,17 @@ PluginInsert::run (BufferSet& bufs, framepos_t start_frame, framepos_t end_frame
 		/* run as normal if we are active or moving from inactive to active */
 
 		if (_session.transport_rolling() || _session.bounce_processing()) {
-			automation_run (bufs, start_frame, end_frame, speed, nframes);
+			automate_and_run (bufs, start_frame, end_frame, speed, nframes);
 		} else {
 			Glib::Threads::Mutex::Lock lm (control_lock(), Glib::Threads::TRY_LOCK);
 			connect_and_run (bufs, start_frame, end_frame, speed, nframes, 0, lm.locked());
 		}
 
 	} else {
+		// XXX should call ::silence() to run plugin(s) for consistent load.
+		// We'll need to change this anyway when bypass can be automated
 		bypass (bufs, nframes);
+		automation_run (start_frame, nframes); // evaluate automation only
 		_delaybuffers.flush ();
 	}
 
@@ -1112,7 +1187,7 @@ PluginInsert::run (BufferSet& bufs, framepos_t start_frame, framepos_t end_frame
 }
 
 void
-PluginInsert::automation_run (BufferSet& bufs, framepos_t start, framepos_t end, double speed, pframes_t nframes)
+PluginInsert::automate_and_run (BufferSet& bufs, framepos_t start, framepos_t end, double speed, pframes_t nframes)
 {
 	Evoral::ControlEvent next_event (0, 0.0f);
 	framecnt_t offset = 0;
@@ -2277,9 +2352,9 @@ PluginInsert::state (bool full)
 {
 	XMLNode& node = Processor::state (full);
 
-	node.add_property("type", _plugins[0]->state_node_name());
-	node.add_property("unique-id", _plugins[0]->unique_id());
-	node.add_property("count", string_compose("%1", _plugins.size()));
+	node.set_property("type", _plugins[0]->state_node_name());
+	node.set_property("unique-id", _plugins[0]->unique_id());
+	node.set_property("count", (uint32_t)_plugins.size());
 
 	/* remember actual i/o configuration (for later placeholder
 	 * in case the plugin goes missing) */
@@ -2289,7 +2364,7 @@ PluginInsert::state (bool full)
 	node.add_child_nocopy (* _preset_out.state (X_("PresetOutput")));
 
 	/* save custom i/o config */
-	node.add_property("custom", _custom_cfg ? "yes" : "no");
+	node.set_property("custom", _custom_cfg);
 	for (uint32_t pc = 0; pc < get_count(); ++pc) {
 		char tmp[128];
 		snprintf (tmp, sizeof(tmp), "InputMap-%d", pc);
@@ -2325,19 +2400,19 @@ PluginInsert::set_control_ids (const XMLNode& node, int version)
 
 	for (iter = nlist.begin(); iter != nlist.end(); ++iter) {
 		if ((*iter)->name() == Controllable::xml_node_name) {
-			XMLProperty const * prop;
 
 			uint32_t p = (uint32_t)-1;
 #ifdef LV2_SUPPORT
-			if ((prop = (*iter)->property (X_("symbol"))) != 0) {
+			std::string str;
+			if ((*iter)->get_property (X_("symbol"), str)) {
 				boost::shared_ptr<LV2Plugin> lv2plugin = boost::dynamic_pointer_cast<LV2Plugin> (_plugins[0]);
 				if (lv2plugin) {
-					p = lv2plugin->port_index(prop->value().c_str());
+					p = lv2plugin->port_index(str.c_str());
 				}
 			}
 #endif
-			if (p == (uint32_t)-1 && (prop = (*iter)->property (X_("parameter"))) != 0) {
-				p = atoi (prop->value());
+			if (p == (uint32_t)-1) {
+				(*iter)->get_property (X_("parameter"), p);
 			}
 
 			if (p != (uint32_t)-1) {
@@ -2366,36 +2441,34 @@ PluginInsert::set_state(const XMLNode& node, int version)
 	XMLNodeList nlist = node.children();
 	XMLNodeIterator niter;
 	XMLPropertyList plist;
-	XMLProperty const * prop;
 	ARDOUR::PluginType type;
 
-	if ((prop = node.property ("type")) == 0) {
+	std::string str;
+	if (!node.get_property ("type", str)) {
 		error << _("XML node describing plugin is missing the `type' field") << endmsg;
 		return -1;
 	}
 
-	if (prop->value() == X_("ladspa") || prop->value() == X_("Ladspa")) { /* handle old school sessions */
+	if (str == X_("ladspa") || str == X_("Ladspa")) { /* handle old school sessions */
 		type = ARDOUR::LADSPA;
-	} else if (prop->value() == X_("lv2")) {
+	} else if (str == X_("lv2")) {
 		type = ARDOUR::LV2;
-	} else if (prop->value() == X_("windows-vst")) {
+	} else if (str == X_("windows-vst")) {
 		type = ARDOUR::Windows_VST;
-	} else if (prop->value() == X_("lxvst")) {
+	} else if (str == X_("lxvst")) {
 		type = ARDOUR::LXVST;
-	} else if (prop->value() == X_("mac-vst")) {
+	} else if (str == X_("mac-vst")) {
 		type = ARDOUR::MacVST;
-	} else if (prop->value() == X_("audiounit")) {
+	} else if (str == X_("audiounit")) {
 		type = ARDOUR::AudioUnit;
-	} else if (prop->value() == X_("luaproc")) {
+	} else if (str == X_("luaproc")) {
 		type = ARDOUR::Lua;
 	} else {
-		error << string_compose (_("unknown plugin type %1 in plugin insert state"),
-				  prop->value())
-		      << endmsg;
+		error << string_compose (_("unknown plugin type %1 in plugin insert state"), str) << endmsg;
 		return -1;
 	}
 
-	prop = node.property ("unique-id");
+	XMLProperty const * prop = node.property ("unique-id");
 
 	if (prop == 0) {
 #ifdef WINDOWS_VST_SUPPORT
@@ -2421,6 +2494,7 @@ PluginInsert::set_state(const XMLNode& node, int version)
 	}
 
 	boost::shared_ptr<Plugin> plugin = find_plugin (_session, prop->value(), type);
+	bool any_vst = false;
 
 	/* treat VST plugins equivalent if they have the same uniqueID
 	 * allow to move sessions windows <> linux */
@@ -2428,6 +2502,7 @@ PluginInsert::set_state(const XMLNode& node, int version)
 	if (plugin == 0 && (type == ARDOUR::Windows_VST || type == ARDOUR::MacVST)) {
 		type = ARDOUR::LXVST;
 		plugin = find_plugin (_session, prop->value(), type);
+		if (plugin) { any_vst = true; }
 	}
 #endif
 
@@ -2435,6 +2510,7 @@ PluginInsert::set_state(const XMLNode& node, int version)
 	if (plugin == 0 && (type == ARDOUR::LXVST || type == ARDOUR::MacVST)) {
 		type = ARDOUR::Windows_VST;
 		plugin = find_plugin (_session, prop->value(), type);
+		if (plugin) { any_vst = true; }
 	}
 #endif
 
@@ -2442,6 +2518,7 @@ PluginInsert::set_state(const XMLNode& node, int version)
 	if (plugin == 0 && (type == ARDOUR::Windows_VST || type == ARDOUR::LXVST)) {
 		type = ARDOUR::MacVST;
 		plugin = find_plugin (_session, prop->value(), type);
+		if (plugin) { any_vst = true; }
 	}
 #endif
 
@@ -2488,9 +2565,7 @@ PluginInsert::set_state(const XMLNode& node, int version)
 		set_control_ids (node, version);
 	}
 
-	if ((prop = node.property ("count")) != 0) {
-		sscanf (prop->value().c_str(), "%u", &count);
-	}
+	node.get_property ("count", count);
 
 	if (_plugins.size() != count) {
 		for (uint32_t n = 1; n < count; ++n) {
@@ -2503,9 +2578,7 @@ PluginInsert::set_state(const XMLNode& node, int version)
 	PBD::ID new_id = this->id();
 	PBD::ID old_id = this->id();
 
-	if ((prop = node.property ("id")) != 0) {
-		old_id = prop->value ();
-	}
+	node.get_property ("id", old_id);
 
 	for (niter = nlist.begin(); niter != nlist.end(); ++niter) {
 
@@ -2513,7 +2586,9 @@ PluginInsert::set_state(const XMLNode& node, int version)
 		   and set all plugins to the same state.
 		*/
 
-		if ((*niter)->name() == plugin->state_node_name()) {
+		if (   ((*niter)->name() == plugin->state_node_name())
+		    || (any_vst && ((*niter)->name() == "lxvst" || (*niter)->name() == "windows-vst" || (*niter)->name() == "mac-vst"))
+		   ) {
 
 			for (Plugins::iterator i = _plugins.begin(); i != _plugins.end(); ++i) {
 				/* Plugin state can include external files which are named after the ID.
@@ -2539,6 +2614,14 @@ PluginInsert::set_state(const XMLNode& node, int version)
 				}
 			}
 
+			/* when copying plugin state, notify UI */
+			for (Controls::const_iterator li = controls().begin(); li != controls().end(); ++li) {
+				boost::shared_ptr<PBD::Controllable> c = boost::dynamic_pointer_cast<PBD::Controllable> (li->second);
+				if (c) {
+					c->Changed (false, Controllable::NoGroup); /* EMIT SIGNAL */
+				}
+			}
+
 			break;
 		}
 	}
@@ -2560,9 +2643,7 @@ PluginInsert::set_state(const XMLNode& node, int version)
 		set_parameter_state_2X (node, version);
 	}
 
-	if ((prop = node.property (X_("custom"))) != 0) {
-		_custom_cfg = string_is_affirmative (prop->value());
-	}
+	node.get_property (X_("custom"), _custom_cfg);
 
 	uint32_t in_maps = 0;
 	uint32_t out_maps = 0;
@@ -2666,10 +2747,8 @@ PluginInsert::set_parameter_state_2X (const XMLNode& node, int version)
 		}
 
 		XMLNodeList cnodes;
-		XMLProperty const * cprop;
 		XMLNodeConstIterator iter;
 		XMLNode *child;
-		const char *port;
 		uint32_t port_id;
 
 		cnodes = (*niter)->children ("port");
@@ -2678,15 +2757,11 @@ PluginInsert::set_parameter_state_2X (const XMLNode& node, int version)
 
 			child = *iter;
 
-			if ((cprop = child->property("number")) != 0) {
-				port = cprop->value().c_str();
-			} else {
+			if (!child->get_property("number", port_id)) {
 				warning << _("PluginInsert: Auto: no ladspa port number") << endmsg;
 				continue;
 			}
 
-			sscanf (port, "%" PRIu32, &port_id);
-
 			if (port_id >= _plugins[0]->parameter_count()) {
 				warning << _("PluginInsert: Auto: port id out of range") << endmsg;
 				continue;
@@ -2698,30 +2773,6 @@ PluginInsert::set_parameter_state_2X (const XMLNode& node, int version)
 			if (c && c->alist()) {
 				if (!child->children().empty()) {
 					c->alist()->set_state (*child->children().front(), version);
-
-					/* In some cases 2.X saves lists with min_yval and max_yval
-					   being FLT_MIN and FLT_MAX respectively.  This causes problems
-					   in A3 because these min/max values are used to compute
-					   where GUI control points should be drawn.  If we see such
-					   values, `correct' them to the min/max of the appropriate
-					   parameter.
-					*/
-
-					float min_y = c->alist()->get_min_y ();
-					float max_y = c->alist()->get_max_y ();
-
-					ParameterDescriptor desc;
-					_plugins.front()->get_parameter_descriptor (port_id, desc);
-
-					if (min_y == FLT_MIN) {
-						min_y = desc.lower;
-					}
-
-					if (max_y == FLT_MAX) {
-						max_y = desc.upper;
-					}
-
-					c->alist()->set_yrange (min_y, max_y);
 				}
 			} else {
 				error << string_compose (_("PluginInsert: automatable control %1 not found - ignored"), port_id) << endmsg;
@@ -2734,6 +2785,16 @@ PluginInsert::set_parameter_state_2X (const XMLNode& node, int version)
 	}
 }
 
+boost::shared_ptr<ReadOnlyControl>
+PluginInsert::control_output (uint32_t num) const
+{
+	CtrlOutMap::const_iterator i = _control_outputs.find (num);
+	if (i == _control_outputs.end ()) {
+		return boost::shared_ptr<ReadOnlyControl> ();
+	} else {
+		return (*i).second;
+	}
+}
 
 string
 PluginInsert::describe_parameter (Evoral::Parameter param)
@@ -2776,7 +2837,6 @@ PluginInsert::PluginControl::PluginControl (PluginInsert*                     p,
 	, _plugin (p)
 {
 	if (alist()) {
-		alist()->reset_default (desc.normal);
 		if (desc.toggled) {
 			list->set_interpolation(Evoral::ControlList::Discrete);
 		}
@@ -2811,15 +2871,12 @@ PluginInsert::PluginControl::catch_up_with_external_value (double user_val)
 XMLNode&
 PluginInsert::PluginControl::get_state ()
 {
-	stringstream ss;
-
 	XMLNode& node (AutomationControl::get_state());
-	ss << parameter().id();
-	node.add_property (X_("parameter"), ss.str());
+	node.set_property (X_("parameter"), parameter().id());
 #ifdef LV2_SUPPORT
 	boost::shared_ptr<LV2Plugin> lv2plugin = boost::dynamic_pointer_cast<LV2Plugin> (_plugin->_plugins[0]);
 	if (lv2plugin) {
-		node.add_property (X_("symbol"), lv2plugin->port_symbol (parameter().id()));
+		node.set_property (X_("symbol"), lv2plugin->port_symbol (parameter().id()));
 	}
 #endif
 
@@ -2846,10 +2903,6 @@ PluginInsert::PluginPropertyControl::PluginPropertyControl (PluginInsert*
 	: AutomationControl (p->session(), param, desc, list)
 	, _plugin (p)
 {
-	if (alist()) {
-		alist()->set_yrange (desc.lower, desc.upper);
-		alist()->reset_default (desc.normal);
-	}
 }
 
 void
@@ -2876,11 +2929,8 @@ PluginInsert::PluginPropertyControl::actually_set_value (double user_val, Contro
 XMLNode&
 PluginInsert::PluginPropertyControl::get_state ()
 {
-	stringstream ss;
-
 	XMLNode& node (AutomationControl::get_state());
-	ss << parameter().id();
-	node.add_property (X_("property"), ss.str());
+	node.set_property (X_("property"), parameter().id());
 	node.remove_property (X_("value"));
 
 	return node;
@@ -2909,6 +2959,7 @@ PluginInsert::get_impulse_analysis_plugin()
 			assert (out == internal_output_streams ());
 		}
 		ret->configure_io (internal_input_streams (), out);
+		ret->set_owner (_owner);
 		_impulseAnalysisPlugin = ret;
 	} else {
 		ret = _impulseAnalysisPlugin.lock();
@@ -3014,19 +3065,21 @@ PluginInsert::latency_changed ()
 void
 PluginInsert::start_touch (uint32_t param_id)
 {
-        boost::shared_ptr<AutomationControl> ac = automation_control (Evoral::Parameter (PluginAutomation, 0, param_id));
-        if (ac) {
-                ac->start_touch (session().audible_frame());
-        }
+	boost::shared_ptr<AutomationControl> ac = automation_control (Evoral::Parameter (PluginAutomation, 0, param_id));
+	if (ac) {
+		// ToDo subtract _plugin_signal_latency  from audible_frame() when rolling, assert > 0
+		ac->start_touch (session().audible_frame());
+	}
 }
 
 void
 PluginInsert::end_touch (uint32_t param_id)
 {
-        boost::shared_ptr<AutomationControl> ac = automation_control (Evoral::Parameter (PluginAutomation, 0, param_id));
-        if (ac) {
-                ac->stop_touch (true, session().audible_frame());
-        }
+	boost::shared_ptr<AutomationControl> ac = automation_control (Evoral::Parameter (PluginAutomation, 0, param_id));
+	if (ac) {
+		// ToDo subtract _plugin_signal_latency  from audible_frame() when rolling, assert > 0
+		ac->stop_touch (session().audible_frame());
+	}
 }
 
 std::ostream& operator<<(std::ostream& o, const ARDOUR::PluginInsert::Match& m)
