@@ -77,6 +77,8 @@ AlsaAudioBackend::AlsaAudioBackend (AudioEngine& e, AudioBackendInfo& info)
 	pthread_mutex_init (&_port_callback_mutex, 0);
 	_input_audio_device_info.valid = false;
 	_output_audio_device_info.valid = false;
+
+	_port_connection_queue.reserve (128);
 }
 
 AlsaAudioBackend::~AlsaAudioBackend ()
@@ -805,7 +807,14 @@ AlsaAudioBackend::_start (bool for_latency_measurement)
 	}
 
 	if (_active || _run) {
-		PBD::error << _("AlsaAudioBackend: already active.") << endmsg;
+		if (for_latency_measurement != _measure_latency) {
+			_measure_latency = for_latency_measurement;
+			update_systemic_audio_latencies();
+			update_systemic_midi_latencies ();
+			PBD::info << _("AlsaAudioBackend: reload latencies.") << endmsg;
+			return NoError;
+		}
+		PBD::info << _("AlsaAudioBackend: already active.") << endmsg;
 		return BackendReinitializationError;
 	}
 
@@ -1043,6 +1052,7 @@ AlsaAudioBackend::stop ()
 	delete _pcmi; _pcmi = 0;
 	_midi_ins = _midi_outs = 0;
 	release_device();
+	_measure_latency = false;
 
 	return (_active == false) ? 0 : -1;
 }
@@ -1119,7 +1129,7 @@ AlsaAudioBackend::create_process_thread (boost::function<void()> func)
 
 	ThreadData* td = new ThreadData (this, func, stacksize);
 
-	if (_realtime_pthread_create (SCHED_FIFO, -21, stacksize,
+	if (_realtime_pthread_create (SCHED_FIFO, -22, stacksize,
 				&thread_id, alsa_process_thread, td)) {
 		pthread_attr_init (&attr);
 		pthread_attr_setstacksize (&attr, stacksize);
@@ -1384,7 +1394,7 @@ AlsaAudioBackend::register_system_audio_ports()
 	const uint32_t lcpp = (_periods_per_cycle - 2) * _samples_per_period;
 
 	/* audio ports */
-	lr.min = lr.max = (_measure_latency ? 0 : _systemic_audio_input_latency);
+	lr.min = lr.max = (_systemic_audio_input_latency);
 	for (int i = 1; i <= a_ins; ++i) {
 		char tmp[64];
 		snprintf(tmp, sizeof(tmp), "system:capture_%d", i);
@@ -1396,7 +1406,7 @@ AlsaAudioBackend::register_system_audio_ports()
 		_system_inputs.push_back (ap);
 	}
 
-	lr.min = lr.max = lcpp + (_measure_latency ? 0 : _systemic_audio_output_latency);
+	lr.min = lr.max = lcpp + (_systemic_audio_output_latency);
 	for (int i = 1; i <= a_out; ++i) {
 		char tmp[64];
 		snprintf(tmp, sizeof(tmp), "system:playback_%d", i);
@@ -1410,6 +1420,23 @@ AlsaAudioBackend::register_system_audio_ports()
 	return 0;
 }
 
+/* libs/ardouralsautil/devicelist.cc appends either of
+ * " (IO)", " (I)", or " (O)"
+ * depending of the device is full-duples or half-duplex
+ */
+static std::string replace_name_io (std::string const& name, bool in)
+{
+	if (name.empty ()) {
+		return "";
+	}
+	size_t pos = name.find_last_of ('(');
+	if (pos == std::string::npos) {
+		assert (0); // this should never happen.
+		return name;
+	}
+	return name.substr (0, pos) + "(" + (in ? "In" : "Out") + ")";
+}
+
 int
 AlsaAudioBackend::register_system_midi_ports(const std::string device)
 {
@@ -1462,11 +1489,11 @@ AlsaAudioBackend::register_system_midi_ports(const std::string device)
 					delete mout;
 				}
 				LatencyRange lr;
-				lr.min = lr.max = (_measure_latency ? 0 : nfo->systemic_output_latency);
+				lr.min = lr.max = (nfo->systemic_output_latency);
 				set_latency_range (p, true, lr);
 				static_cast<AlsaMidiPort*>(p)->set_n_periods(_periods_per_cycle); // TODO check MIDI alignment
 				AlsaPort *ap = static_cast<AlsaPort*>(p);
-				ap->set_pretty_name (i->first);
+				ap->set_pretty_name (replace_name_io (i->first, false));
 				_system_midi_out.push_back (ap);
 				_rmidi_out.push_back (mout);
 			}
@@ -1502,10 +1529,10 @@ AlsaAudioBackend::register_system_midi_ports(const std::string device)
 					continue;
 				}
 				LatencyRange lr;
-				lr.min = lr.max = (_measure_latency ? 0 : nfo->systemic_input_latency);
+				lr.min = lr.max = (nfo->systemic_input_latency);
 				set_latency_range (p, false, lr);
 				AlsaPort *ap = static_cast<AlsaPort*>(p);
-				ap->set_pretty_name (i->first);
+				ap->set_pretty_name (replace_name_io (i->first, true));
 				_system_midi_in.push_back (ap);
 				_rmidi_in.push_back (midin);
 			}
@@ -1660,7 +1687,7 @@ AlsaAudioBackend::get_connections (PortEngine::PortHandle port, std::vector<std:
 int
 AlsaAudioBackend::midi_event_get (
 		pframes_t& timestamp,
-		size_t& size, uint8_t** buf, void* port_buffer,
+		size_t& size, uint8_t const** buf, void* port_buffer,
 		uint32_t event_index)
 {
 	assert (buf && port_buffer);
@@ -1668,11 +1695,11 @@ AlsaAudioBackend::midi_event_get (
 	if (event_index >= source.size ()) {
 		return -1;
 	}
-	AlsaMidiEvent * const event = source[event_index].get ();
+	AlsaMidiEvent const& event = source[event_index];
 
-	timestamp = event->timestamp ();
-	size = event->size ();
-	*buf = event->data ();
+	timestamp = event.timestamp ();
+	size = event.size ();
+	*buf = event.data ();
 	return 0;
 }
 
@@ -1683,15 +1710,18 @@ AlsaAudioBackend::midi_event_put (
 		const uint8_t* buffer, size_t size)
 {
 	assert (buffer && port_buffer);
+	if (size >= MaxAlsaMidiEventSize) {
+		return -1;
+	}
 	AlsaMidiBuffer& dst = * static_cast<AlsaMidiBuffer*>(port_buffer);
-	if (dst.size () && (pframes_t)dst.back ()->timestamp () > timestamp) {
 #ifndef NDEBUG
+	if (dst.size () && (pframes_t)dst.back ().timestamp () > timestamp) {
 		// nevermind, ::get_buffer() sorts events
 		fprintf (stderr, "AlsaMidiBuffer: it's too late for this event. %d > %d\n",
-				(pframes_t)dst.back ()->timestamp (), timestamp);
-#endif
+				(pframes_t)dst.back ().timestamp (), timestamp);
 	}
-	dst.push_back (boost::shared_ptr<AlsaMidiEvent>(new AlsaMidiEvent (timestamp, buffer, size)));
+#endif
+	dst.push_back (AlsaMidiEvent (timestamp, buffer, size));
 	return 0;
 }
 
@@ -1924,7 +1954,7 @@ AlsaAudioBackend::main_process_thread ()
 					AlsaMidiIn *rm = _rmidi_in.at(i);
 					void *bptr = (*it)->get_buffer(0);
 					pframes_t time;
-					uint8_t data[64]; // match MaxAlsaEventSize in alsa_rawmidi.cc
+					uint8_t data[MaxAlsaMidiEventSize];
 					size_t size = sizeof(data);
 					midi_clear(bptr);
 					while (rm->recv_event (time, data, size)) {
@@ -1958,7 +1988,7 @@ AlsaAudioBackend::main_process_thread ()
 					AlsaMidiOut *rm = _rmidi_out.at(i);
 					rm->sync_time (clock1);
 					for (AlsaMidiBuffer::const_iterator mit = src->begin (); mit != src->end (); ++mit) {
-						rm->send_event ((*mit)->timestamp(), (*mit)->data(), (*mit)->size());
+						rm->send_event (mit->timestamp (), mit->data (), mit->size ());
 					}
 				}
 
@@ -2296,13 +2326,18 @@ AlsaMidiPort::AlsaMidiPort (AlsaAudioBackend &b, const std::string& name, PortFl
 {
 	_buffer[0].clear ();
 	_buffer[1].clear ();
+	_buffer[2].clear ();
+
+	_buffer[0].reserve(256);
+	_buffer[1].reserve(256);
+	_buffer[2].reserve(256);
 }
 
 AlsaMidiPort::~AlsaMidiPort () { }
 
 struct MidiEventSorter {
-	bool operator() (const boost::shared_ptr<AlsaMidiEvent>& a, const boost::shared_ptr<AlsaMidiEvent>& b) {
-		return *a < *b;
+	bool operator() (AlsaMidiEvent const& a, AlsaMidiEvent const& b) {
+		return a < b;
 	}
 };
 
@@ -2316,10 +2351,10 @@ void* AlsaMidiPort::get_buffer (pframes_t /* nframes */)
 				++i) {
 			const AlsaMidiBuffer * src = static_cast<const AlsaMidiPort*>(*i)->const_buffer ();
 			for (AlsaMidiBuffer::const_iterator it = src->begin (); it != src->end (); ++it) {
-				(_buffer[_bufperiod]).push_back (boost::shared_ptr<AlsaMidiEvent>(new AlsaMidiEvent (**it)));
+				(_buffer[_bufperiod]).push_back (*it);
 			}
 		}
-		std::sort ((_buffer[_bufperiod]).begin (), (_buffer[_bufperiod]).end (), MidiEventSorter());
+		std::stable_sort ((_buffer[_bufperiod]).begin (), (_buffer[_bufperiod]).end (), MidiEventSorter());
 	}
 	return &(_buffer[_bufperiod]);
 }
@@ -2327,10 +2362,8 @@ void* AlsaMidiPort::get_buffer (pframes_t /* nframes */)
 AlsaMidiEvent::AlsaMidiEvent (const pframes_t timestamp, const uint8_t* data, size_t size)
 	: _size (size)
 	, _timestamp (timestamp)
-	, _data (0)
 {
-	if (size > 0) {
-		_data = (uint8_t*) malloc (size);
+	if (size > 0 && size < MaxAlsaMidiEventSize) {
 		memcpy (_data, data, size);
 	}
 }
@@ -2338,14 +2371,9 @@ AlsaMidiEvent::AlsaMidiEvent (const pframes_t timestamp, const uint8_t* data, si
 AlsaMidiEvent::AlsaMidiEvent (const AlsaMidiEvent& other)
 	: _size (other.size ())
 	, _timestamp (other.timestamp ())
-	, _data (0)
 {
-	if (other.size () && other.const_data ()) {
-		_data = (uint8_t*) malloc (other.size ());
-		memcpy (_data, other.const_data (), other.size ());
+	if (other._size > 0) {
+		assert (other._size < MaxAlsaMidiEventSize);
+		memcpy (_data, other._data, other._size);
 	}
 };
-
-AlsaMidiEvent::~AlsaMidiEvent () {
-	free (_data);
-};
